# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccessScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AccessScope enums."""

    JOB = "job"
    """Grants access to perform all operations on the Job containing the Task."""


class AllocationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AllocationState enums."""

    STEADY = "steady"
    """The Pool is not resizing. There are no changes to the number of Compute Nodes in the Pool in
    progress. A Pool enters this state when it is created and when no operations are being
    performed on the Pool to change the number of Compute Nodes."""
    RESIZING = "resizing"
    """The Pool is resizing; that is, Compute Nodes are being added to or removed from the Pool."""
    STOPPING = "stopping"
    """The Pool was resizing, but the user has requested that the resize be stopped, but the stop
    request has not yet been completed."""


class AutoUserScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AutoUserScope enums."""

    TASK = "task"
    """Specifies that the service should create a new user for the Task."""
    POOL = "pool"
    """Specifies that the Task runs as the common auto user Account which is created on every Compute
    Node in a Pool."""


class BatchCertificateFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchCertificateFormat enums."""

    PFX = "pfx"
    """The Certificate is a PFX (PKCS#12) formatted Certificate or Certificate chain."""
    CER = "cer"
    """The Certificate is a base64-encoded X.509 Certificate."""


class BatchCertificateState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchCertificateState enums."""

    ACTIVE = "active"
    """The Certificate is available for use in Pools."""
    DELETING = "deleting"
    """The user has requested that the Certificate be deleted, but the delete operation has not yet
    completed. You may not reference the Certificate when creating or updating Pools."""
    DELETE_FAILED = "deletefailed"
    """The user requested that the Certificate be deleted, but there are Pools that still have
    references to the Certificate, or it is still installed on one or more Nodes. (The latter can
    occur if the Certificate has been removed from the Pool, but the Compute Node has not yet
    restarted. Compute Nodes refresh their Certificates only when they restart.) You may use the
    cancel Certificate delete operation to cancel the delete, or the delete Certificate operation
    to retry the delete."""


class BatchCertificateStoreLocation(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchCertificateStoreLocation enums."""

    CURRENT_USER = "currentuser"
    """Certificates should be installed to the CurrentUser Certificate store."""
    LOCAL_MACHINE = "localmachine"
    """Certificates should be installed to the LocalMachine Certificate store."""


class BatchCertificateVisibility(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchCertificateVisibility enums."""

    START_TASK = "starttask"
    """The Certificate should be visible to the user account under which the StartTask is run. Note
    that if AutoUser Scope is Pool for both the StartTask and a Task, this certificate will be
    visible to the Task as well."""
    TASK = "task"
    """The Certificate should be visible to the user accounts under which Job Tasks are run."""
    REMOTE_USER = "remoteuser"
    """The Certificate should be visible to the user accounts under which users remotely access the
    Compute Node."""


class BatchJobAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchJobAction enums."""

    NONE = "none"
    """Take no action."""
    DISABLE = "disable"
    """Disable the Job. This is equivalent to calling the disable Job API, with a disableTasks value
    of requeue."""
    TERMINATE = "terminate"
    """Terminate the Job. The terminationReason in the Job's executionInfo is set to "TaskFailed"."""


class BatchJobPreparationTaskState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchJobPreparationTaskState enums."""

    RUNNING = "running"
    """The Task is currently running (including retrying)."""
    COMPLETED = "completed"
    """The Task has exited with exit code 0, or the Task has exhausted its retry limit, or the Batch
    service was unable to start the Task due to Task preparation errors (such as resource file
    download failures)."""


class BatchJobReleaseTaskState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchJobReleaseTaskState enums."""

    RUNNING = "running"
    """The Task is currently running (including retrying)."""
    COMPLETED = "completed"
    """The Task has exited with exit code 0, or the Task has exhausted its retry limit, or the Batch
    service was unable to start the Task due to Task preparation errors (such as resource file
    download failures)."""


class BatchJobScheduleState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchJobScheduleState enums."""

    ACTIVE = "active"
    """The Job Schedule is active and will create Jobs as per its schedule."""
    COMPLETED = "completed"
    """The Job Schedule has terminated, either by reaching its end time or by the user terminating it
    explicitly."""
    DISABLED = "disabled"
    """The user has disabled the Job Schedule. The scheduler will not initiate any new Jobs will on
    this schedule, but any existing active Job will continue to run."""
    TERMINATING = "terminating"
    """The Job Schedule has no more work to do, or has been explicitly terminated by the user, but the
    termination operation is still in progress. The scheduler will not initiate any new Jobs for
    this Job Schedule, nor is any existing Job active."""
    DELETING = "deleting"
    """The user has requested that the Job Schedule be deleted, but the delete operation is still in
    progress. The scheduler will not initiate any new Jobs for this Job Schedule, and will delete
    any existing Jobs and Tasks under the Job Schedule, including any active Job. The Job Schedule
    will be deleted when all Jobs and Tasks under the Job Schedule have been deleted."""


class BatchJobState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchJobState enums."""

    ACTIVE = "active"
    """The Job is available to have Tasks scheduled."""
    DISABLING = "disabling"
    """A user has requested that the Job be disabled, but the disable operation is still in progress
    (for example, waiting for Tasks to terminate)."""
    DISABLED = "disabled"
    """A user has disabled the Job. No Tasks are running, and no new Tasks will be scheduled."""
    ENABLING = "enabling"
    """A user has requested that the Job be enabled, but the enable operation is still in progress."""
    TERMINATING = "terminating"
    """The Job is about to complete, either because a Job Manager Task has completed or because the
    user has terminated the Job, but the terminate operation is still in progress (for example,
    because Job Release Tasks are running)."""
    COMPLETED = "completed"
    """All Tasks have terminated, and the system will not accept any more Tasks or any further changes
    to the Job."""
    DELETING = "deleting"
    """A user has requested that the Job be deleted, but the delete operation is still in progress
    (for example, because the system is still terminating running Tasks)."""


class BatchNodeCommunicationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeCommunicationMode enums."""

    DEFAULT = "default"
    """The node communication mode is automatically set by the Batch service."""
    CLASSIC = "classic"
    """Nodes using the classic communication mode require inbound TCP communication on ports 29876 and
    29877 from the "BatchNodeManagement.{region}" service tag and outbound TCP communication on
    port 443 to the "Storage.region" and "BatchNodeManagement.{region}" service tags."""
    SIMPLIFIED = "simplified"
    """Nodes using the simplified communication mode require outbound TCP communication on port 443 to
    the "BatchNodeManagement.{region}" service tag. No open inbound ports are required."""


class BatchNodeDeallocateOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeDeallocateOption enums."""

    REQUEUE = "requeue"
    """Terminate running Task processes and requeue the Tasks. The Tasks will run again when a Compute
    Node is available. Deallocate the Compute Node as soon as Tasks have been terminated."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again. Deallocate the Compute Node as soon as Tasks have been
    terminated."""
    TASK_COMPLETION = "taskcompletion"
    """Allow currently running Tasks to complete. Schedule no new Tasks while waiting. Deallocate the
    Compute Node when all Tasks have completed."""
    RETAINED_DATA = "retaineddata"
    """Allow currently running Tasks to complete, then wait for all Task data retention periods to
    expire. Schedule no new Tasks while waiting. Deallocate the Compute Node when all Task
    retention periods have expired."""


class BatchNodeDeallocationOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeDeallocationOption enums."""

    REQUEUE = "requeue"
    """Terminate running Task processes and requeue the Tasks. The Tasks will run again when a Compute
    Node is available. Remove Compute Nodes as soon as Tasks have been terminated."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again. Remove Compute Nodes as soon as Tasks have been terminated."""
    TASK_COMPLETION = "taskcompletion"
    """Allow currently running Tasks to complete. Schedule no new Tasks while waiting. Remove Compute
    Nodes when all Tasks have completed."""
    RETAINED_DATA = "retaineddata"
    """Allow currently running Tasks to complete, then wait for all Task data retention periods to
    expire. Schedule no new Tasks while waiting. Remove Compute Nodes when all Task retention
    periods have expired."""


class BatchNodeDisableSchedulingOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeDisableSchedulingOption enums."""

    REQUEUE = "requeue"
    """Terminate running Task processes and requeue the Tasks. The Tasks may run again on other
    Compute Nodes, or when Task scheduling is re-enabled on this Compute Node. Enter offline state
    as soon as Tasks have been terminated."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again. Enter offline state as soon as Tasks have been terminated."""
    TASK_COMPLETION = "taskcompletion"
    """Allow currently running Tasks to complete. Schedule no new Tasks while waiting. Enter offline
    state when all Tasks have completed."""


class BatchNodeFillType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeFillType enums."""

    SPREAD = "spread"
    """Tasks should be assigned evenly across all Compute Nodes in the Pool."""
    PACK = "pack"
    """As many Tasks as possible (taskSlotsPerNode) should be assigned to each Compute Node in the
    Pool before any Tasks are assigned to the next Compute Node in the Pool."""


class BatchNodePlacementPolicyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodePlacementPolicyType enums."""

    REGIONAL = "regional"
    """All nodes in the pool will be allocated in the same region."""
    ZONAL = "zonal"
    """Nodes in the pool will be spread across different availability zones with best effort
    balancing."""


class BatchNodeRebootOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeRebootOption enums."""

    REQUEUE = "requeue"
    """Terminate running Task processes and requeue the Tasks. The Tasks will run again when a Compute
    Node is available. Restart the Compute Node as soon as Tasks have been terminated."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again. Restart the Compute Node as soon as Tasks have been
    terminated."""
    TASK_COMPLETION = "taskcompletion"
    """Allow currently running Tasks to complete. Schedule no new Tasks while waiting. Restart the
    Compute Node when all Tasks have completed."""
    RETAINED_DATA = "retaineddata"
    """Allow currently running Tasks to complete, then wait for all Task data retention periods to
    expire. Schedule no new Tasks while waiting. Restart the Compute Node when all Task retention
    periods have expired."""


class BatchNodeReimageOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeReimageOption enums."""

    REQUEUE = "requeue"
    """Terminate running Task processes and requeue the Tasks. The Tasks will run again when a Compute
    Node is available. Reimage the Compute Node as soon as Tasks have been terminated."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again. Reimage the Compute Node as soon as Tasks have been
    terminated."""
    TASK_COMPLETION = "taskcompletion"
    """Allow currently running Tasks to complete. Schedule no new Tasks while waiting. Reimage the
    Compute Node when all Tasks have completed."""
    RETAINED_DATA = "retaineddata"
    """Allow currently running Tasks to complete, then wait for all Task data retention periods to
    expire. Schedule no new Tasks while waiting. Reimage the Compute Node when all Task retention
    periods have expired."""


class BatchNodeState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchNodeState enums."""

    IDLE = "idle"
    """The Compute Node is not currently running a Task."""
    REBOOTING = "rebooting"
    """The Compute Node is rebooting."""
    REIMAGING = "reimaging"
    """The Compute Node is reimaging."""
    RUNNING = "running"
    """The Compute Node is running one or more Tasks (other than a StartTask)."""
    UNUSABLE = "unusable"
    """The Compute Node cannot be used for Task execution due to errors."""
    CREATING = "creating"
    """The Batch service has obtained the underlying virtual machine from Azure Compute, but it has
    not yet started to join the Pool."""
    STARTING = "starting"
    """The Batch service is starting on the underlying virtual machine."""
    WAITING_FOR_START_TASK = "waitingforstarttask"
    """The StartTask has started running on the Compute Node, but waitForSuccess is set and the
    StartTask has not yet completed."""
    START_TASK_FAILED = "starttaskfailed"
    """The StartTask has failed on the Compute Node (and exhausted all retries), and waitForSuccess is
    set. The Compute Node is not usable for running Tasks."""
    UNKNOWN = "unknown"
    """The Batch service has lost contact with the Compute Node, and does not know its true state."""
    LEAVING_POOL = "leavingpool"
    """The Compute Node is leaving the Pool, either because the user explicitly removed it or because
    the Pool is resizing or autoscaling down."""
    OFFLINE = "offline"
    """The Compute Node is not currently running a Task, and scheduling of new Tasks to the Compute
    Node is disabled."""
    PREEMPTED = "preempted"
    """The Spot/Low-priority Compute Node has been preempted. Tasks which were running on the Compute
    Node when it was preempted will be rescheduled when another Compute Node becomes available."""
    UPGRADING_OS = "upgradingos"
    """The Compute Node is undergoing an OS upgrade operation."""
    DEALLOCATED = "deallocated"
    """The Compute Node is deallocated."""
    DEALLOCATING = "deallocating"
    """The Compute Node is deallocating."""


class BatchPoolIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchPoolIdentityType enums."""

    USER_ASSIGNED = "UserAssigned"
    """Batch pool has user assigned identities with it."""
    NONE = "None"
    """Batch pool has no identity associated with it. Setting ``None`` in update pool will remove
    existing identities."""


class BatchPoolLifetimeOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchPoolLifetimeOption enums."""

    JOB_SCHEDULE = "jobschedule"
    """The Pool exists for the lifetime of the Job Schedule. The Batch Service creates the Pool when
    it creates the first Job on the schedule. You may apply this option only to Job Schedules, not
    to Jobs."""
    JOB = "job"
    """The Pool exists for the lifetime of the Job to which it is dedicated. The Batch service creates
    the Pool when it creates the Job. If the 'job' option is applied to a Job Schedule, the Batch
    service creates a new auto Pool for every Job created on the schedule."""


class BatchPoolState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchPoolState enums."""

    ACTIVE = "active"
    """The Pool is available to run Tasks subject to the availability of Compute Nodes."""
    DELETING = "deleting"
    """The user has requested that the Pool be deleted, but the delete operation has not yet
    completed."""


class BatchStartTaskState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchStartTaskState enums."""

    RUNNING = "running"
    """The StartTask is currently running."""
    COMPLETED = "completed"
    """The StartTask has exited with exit code 0, or the StartTask has failed and the retry limit has
    reached, or the StartTask process did not run due to Task preparation errors (such as resource
    file download failures)."""


class BatchSubtaskState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchSubtaskState enums."""

    PREPARING = "preparing"
    """The Task has been assigned to a Compute Node, but is waiting for a required Job Preparation
    Task to complete on the Compute Node. If the Job Preparation Task succeeds, the Task will move
    to running. If the Job Preparation Task fails, the Task will return to active and will be
    eligible to be assigned to a different Compute Node."""
    RUNNING = "running"
    """The Task is running on a Compute Node. This includes task-level preparation such as downloading
    resource files or deploying Packages specified on the Task - it does not necessarily mean that
    the Task command line has started executing."""
    COMPLETED = "completed"
    """The Task is no longer eligible to run, usually because the Task has finished successfully, or
    the Task has finished unsuccessfully and has exhausted its retry limit. A Task is also marked
    as completed if an error occurred launching the Task, or when the Task has been terminated."""


class BatchTaskAddStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchTaskAddStatus enums."""

    SUCCESS = "success"
    """The Task was added successfully."""
    CLIENT_ERROR = "clienterror"
    """The Task failed to add due to a client error and should not be retried without modifying the
    request as appropriate."""
    SERVER_ERROR = "servererror"
    """Task failed to add due to a server error and can be retried without modification."""


class BatchTaskExecutionResult(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchTaskExecutionResult enums."""

    SUCCESS = "success"
    """The Task ran successfully."""
    FAILURE = "failure"
    """There was an error during processing of the Task. The failure may have occurred before the Task
    process was launched, while the Task process was executing, or after the Task process exited."""


class BatchTaskState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BatchTaskState enums."""

    ACTIVE = "active"
    """The Task is queued and able to run, but is not currently assigned to a Compute Node. A Task
    enters this state when it is created, when it is enabled after being disabled, or when it is
    awaiting a retry after a failed run."""
    PREPARING = "preparing"
    """The Task has been assigned to a Compute Node, but is waiting for a required Job Preparation
    Task to complete on the Compute Node. If the Job Preparation Task succeeds, the Task will move
    to running. If the Job Preparation Task fails, the Task will return to active and will be
    eligible to be assigned to a different Compute Node."""
    RUNNING = "running"
    """The Task is running on a Compute Node. This includes task-level preparation such as downloading
    resource files or deploying Packages specified on the Task - it does not necessarily mean that
    the Task command line has started executing."""
    COMPLETED = "completed"
    """The Task is no longer eligible to run, usually because the Task has finished successfully, or
    the Task has finished unsuccessfully and has exhausted its retry limit. A Task is also marked
    as completed if an error occurred launching the Task, or when the Task has been terminated."""


class CachingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CachingType enums."""

    NONE = "none"
    """The caching mode for the disk is not enabled."""
    READ_ONLY = "readonly"
    """The caching mode for the disk is read only."""
    READ_WRITE = "readwrite"
    """The caching mode for the disk is read and write."""


class ContainerHostDataPath(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The paths which will be mounted to container task's container."""

    SHARED = "Shared"
    """The path for multi-instances task to shared their files."""
    STARTUP = "Startup"
    """The path for start task."""
    VFS_MOUNTS = "VfsMounts"
    """The path contains all virtual file systems are mounted on this node."""
    TASK = "Task"
    """The task path."""
    JOB_PREP = "JobPrep"
    """The job-prep task path."""
    APPLICATIONS = "Applications"
    """The applications path."""


class ContainerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ContainerType enums."""

    DOCKER_COMPATIBLE = "dockerCompatible"
    """A Docker compatible container technology will be used to launch the containers."""
    CRI_COMPATIBLE = "criCompatible"
    """A CRI based technology will be used to launch the containers."""


class ContainerWorkingDirectory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ContainerWorkingDirectory enums."""

    TASK_WORKING_DIRECTORY = "taskWorkingDirectory"
    """Use the standard Batch service Task working directory, which will contain the Task Resource
    Files populated by Batch."""
    CONTAINER_IMAGE_DEFAULT = "containerImageDefault"
    """Use the working directory defined in the container Image. Beware that this directory will not
    contain the Resource Files downloaded by Batch."""


class DependencyAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """DependencyAction enums."""

    SATISFY = "satisfy"
    """Satisfy tasks waiting on this task; once all dependencies are satisfied, the task will be
    scheduled to run."""
    BLOCK = "block"
    """Blocks tasks waiting on this task, preventing them from being scheduled."""


class DiffDiskPlacement(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the ephemeral disk placement for operating system disk for all compute nodes (VMs) in
    the pool.  This property can be used by user in the request to choose which location the
    operating system should be in. e.g., cache disk space for Ephemeral OS disk provisioning. For
    more information on Ephemeral OS disk size requirements, please refer to Ephemeral OS disk size
    requirements for Windows VMs at
    `https://learn.microsoft.com/azure/virtual-machines/windows/ephemeral-os-disks#size-requirements
    <https://learn.microsoft.com/azure/virtual-machines/windows/ephemeral-os-disks#size-requirements>`_
    and Linux VMs at
    `https://learn.microsoft.com/azure/virtual-machines/linux/ephemeral-os-disks#size-requirements
    <https://learn.microsoft.com/azure/virtual-machines/linux/ephemeral-os-disks#size-requirements>`_.
    """

    CACHE_DISK = "cachedisk"
    """The Ephemeral OS Disk is stored on the VM cache."""


class DisableBatchJobOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """DisableBatchJobOption enums."""

    REQUEUE = "requeue"
    """Terminate running Tasks and requeue them. The Tasks will run again when the Job is enabled."""
    TERMINATE = "terminate"
    """Terminate running Tasks. The Tasks will be completed with failureInfo indicating that they were
    terminated, and will not run again."""
    WAIT = "wait"
    """Allow currently running Tasks to complete."""


class DiskEncryptionTarget(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """DiskEncryptionTarget enums."""

    OS_DISK = "osdisk"
    """The OS Disk on the compute node is encrypted."""
    TEMPORARY_DISK = "temporarydisk"
    """The temporary disk on the compute node is encrypted. On Linux this encryption applies to other
    partitions (such as those on mounted data disks) when encryption occurs at boot time."""


class DynamicVNetAssignmentScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """DynamicVNetAssignmentScope enums."""

    NONE = "none"
    """No dynamic VNet assignment is enabled."""
    JOB = "job"
    """Dynamic VNet assignment is done per-job."""


class ElevationLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ElevationLevel enums."""

    NON_ADMIN = "nonadmin"
    """The user is a standard user without elevated access."""
    ADMIN = "admin"
    """The user is a user with elevated access and operates with full Administrator permissions."""


class ErrorCategory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ErrorCategory enums."""

    USER_ERROR = "usererror"
    """The error is due to a user issue, such as misconfiguration."""
    SERVER_ERROR = "servererror"
    """The error is due to an internal server issue."""


class ImageVerificationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ImageVerificationType enums."""

    VERIFIED = "verified"
    """The Image is guaranteed to be compatible with the associated Compute Node agent SKU and all
    Batch features have been confirmed to work as expected."""
    UNVERIFIED = "unverified"
    """The associated Compute Node agent SKU should have binary compatibility with the Image, but
    specific functionality has not been verified."""


class InboundEndpointProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InboundEndpointProtocol enums."""

    TCP = "tcp"
    """Use TCP for the endpoint."""
    UDP = "udp"
    """Use UDP for the endpoint."""


class IpAddressProvisioningType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """IPAddressProvisioningType enums."""

    BATCH_MANAGED = "batchmanaged"
    """A public IP will be created and managed by Batch. There may be multiple public IPs depending on
    the size of the Pool."""
    USER_MANAGED = "usermanaged"
    """Public IPs are provided by the user and will be used to provision the Compute Nodes."""
    NO_PUBLIC_IP_ADDRESSES = "nopublicipaddresses"
    """No public IP Address will be created."""


class LoginMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """LoginMode enums."""

    BATCH = "batch"
    """The LOGON32_LOGON_BATCH Win32 login mode. The batch login mode is recommended for long running
    parallel processes."""
    INTERACTIVE = "interactive"
    """The LOGON32_LOGON_INTERACTIVE Win32 login mode. UAC is enabled on Windows
    VirtualMachineConfiguration Pools. If this option is used with an elevated user identity in a
    Windows VirtualMachineConfiguration Pool, the user session will not be elevated unless the
    application executed by the Task command line is configured to always require administrative
    privilege or to always require maximum privilege."""


class NetworkSecurityGroupRuleAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """NetworkSecurityGroupRuleAccess enums."""

    ALLOW = "allow"
    """Allow access."""
    DENY = "deny"
    """Deny access."""


class OnAllBatchTasksComplete(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The action the Batch service should take when all Tasks in the Job are in the completed state."""

    NO_ACTION = "noaction"
    """Do nothing. The Job remains active unless terminated or disabled by some other means."""
    TERMINATE_JOB = "terminatejob"
    """Terminate the Job. The Job's terminationReason is set to 'AllTasksComplete'."""


class OnBatchTaskFailure(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """OnTaskFailure enums."""

    NO_ACTION = "noaction"
    """Do nothing. The Job remains active unless terminated or disabled by some other means."""
    PERFORM_EXIT_OPTIONS_JOB_ACTION = "performexitoptionsjobaction"
    """Terminate the Job. The Job's terminationReason is set to 'AllTasksComplete'."""


class OSType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """OSType enums."""

    LINUX = "linux"
    """The Linux operating system."""
    WINDOWS = "windows"
    """The Windows operating system."""


class OutputFileUploadCondition(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """OutputFileUploadCondition enums."""

    TASK_SUCCESS = "tasksuccess"
    """Upload the file(s) only after the Task process exits with an exit code of 0."""
    TASK_FAILURE = "taskfailure"
    """Upload the file(s) only after the Task process exits with a nonzero exit code."""
    TASK_COMPLETION = "taskcompletion"
    """Upload the file(s) after the Task process exits, no matter what the exit code was."""


class SchedulingState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SchedulingState enums."""

    ENABLED = "enabled"
    """Tasks can be scheduled on the Compute Node."""
    DISABLED = "disabled"
    """No new Tasks will be scheduled on the Compute Node. Tasks already running on the Compute Node
    may still run to completion. All Compute Nodes start with scheduling enabled."""


class SecurityEncryptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SecurityEncryptionTypes enums."""

    NON_PERSISTED_TPM = "NonPersistedTPM"
    """NonPersistedTPM"""
    VM_GUEST_STATE_ONLY = "VMGuestStateOnly"
    """VMGuestStateOnly"""


class SecurityTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the SecurityType of the virtual machine. It has to be set to any specified value to
    enable UefiSettings.
    """

    TRUSTED_LAUNCH = "trustedLaunch"
    """Trusted launch protects against advanced and persistent attack techniques."""
    CONFIDENTIAL_VM = "confidentialVM"
    """Azure confidential computing offers confidential VMs are for tenants with high security and
    confidentiality requirements. These VMs provide a strong, hardware-enforced boundary to help
    meet your security needs. You can use confidential VMs for migrations without making changes to
    your code, with the platform protecting your VM's state from being read or modified."""


class StatusLevelTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Level code."""

    ERROR = "Error"
    """Error"""
    INFO = "Info"
    """Info"""
    WARNING = "Warning"
    """Warning"""


class StorageAccountType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """StorageAccountType enums."""

    STANDARD_LRS = "standard_lrs"
    """The data disk should use standard locally redundant storage."""
    PREMIUM_LRS = "premium_lrs"
    """The data disk should use premium locally redundant storage."""
    STANDARD_SSDLRS = "standardssd_lrs"
    """The data disk / OS disk should use standard SSD locally redundant storage."""


class UpgradeMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """UpgradeMode enums."""

    AUTOMATIC = "automatic"
    """TAll virtual machines in the scale set are automatically updated at the same time."""
    MANUAL = "manual"
    """You control the application of updates to virtual machines in the scale set. You do this by
    using the manualUpgrade action."""
    ROLLING = "rolling"
    """The existing instances in a scale set are brought down in batches to be upgraded. Once the
    upgraded batch is complete, the instances will begin taking traffic again and the next batch
    will begin. This continues until all instances brought up-to-date."""



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import time
import unittest
from unittest.mock import Mock
from azure.appconfiguration.provider import WatchKey
from devtools_testutils import recorded_by_proxy
from preparers import app_config_decorator_aad
from testcase import AppConfigTestCase, has_feature_flag
from test_constants import FEATURE_MANAGEMENT_KEY


class TestAppConfigurationProvider(AppConfigTestCase, unittest.TestCase):
    # method: refresh
    @recorded_by_proxy
    @app_config_decorator_aad
    def test_refresh(self, appconfiguration_endpoint_string, appconfiguration_keyvault_secret_url):
        mock_callback = Mock()
        client = self.create_aad_client(
            appconfiguration_endpoint_string,
            keyvault_secret_url=appconfiguration_keyvault_secret_url,
            refresh_on=[WatchKey("refresh_message")],
            refresh_interval=1,
            on_refresh_success=mock_callback,
            feature_flag_enabled=True,
            feature_flag_refresh_enabled=True,
        )
        assert client["refresh_message"] == "original value"
        assert client["my_json"]["key"] == "value"
        assert FEATURE_MANAGEMENT_KEY in client
        assert has_feature_flag(client, "Alpha")

        appconfig_client = self.create_aad_sdk_client(appconfiguration_endpoint_string)

        setting = appconfig_client.get_configuration_setting(key="refresh_message")
        setting.value = "updated value"
        feature_flag = appconfig_client.get_configuration_setting(key=".appconfig.featureflag/Alpha")
        feature_flag.enabled = True
        appconfig_client.set_configuration_setting(setting)
        appconfig_client.set_configuration_setting(feature_flag)

        # Waiting for the refresh interval to pass
        time.sleep(2)

        client.refresh()
        assert client["refresh_message"] == "updated value"
        assert has_feature_flag(client, "Alpha", True)
        assert mock_callback.call_count == 1

        setting.value = "original value"
        feature_flag.enabled = False
        appconfig_client.set_configuration_setting(setting)
        appconfig_client.set_configuration_setting(feature_flag)

        # Waiting for the refresh interval to pass
        time.sleep(2)

        client.refresh()
        assert client["refresh_message"] == "original value"
        assert has_feature_flag(client, "Alpha", False)
        assert mock_callback.call_count == 2

        setting.value = "updated value 2"
        feature_flag.enabled = True
        appconfig_client.set_configuration_setting(setting)
        appconfig_client.set_configuration_setting(feature_flag)

        # Not waiting for the refresh interval to pass
        client.refresh()
        assert client["refresh_message"] == "original value"
        assert has_feature_flag(client, "Alpha", False)
        assert mock_callback.call_count == 2

        setting.value = "original value"
        appconfig_client.set_configuration_setting(setting)

        client.refresh()
        assert client["refresh_message"] == "original value"
        assert mock_callback.call_count == 2

    # method: refresh
    @recorded_by_proxy
    @app_config_decorator_aad
    def test_empty_refresh(self, appconfiguration_endpoint_string, appconfiguration_keyvault_secret_url):
        mock_callback = Mock()
        client = self.create_aad_client(
            appconfiguration_endpoint_string,
            keyvault_secret_url=appconfiguration_keyvault_secret_url,
            on_refresh_success=mock_callback,
            feature_flag_enabled=True,
        )
        assert client["refresh_message"] == "original value"
        assert client["non_refreshed_message"] == "Static"
        assert client["my_json"]["key"] == "value"
        assert FEATURE_MANAGEMENT_KEY in client
        assert has_feature_flag(client, "Alpha")

        appconfig_client = self.create_aad_sdk_client(appconfiguration_endpoint_string)

        setting = appconfig_client.get_configuration_setting(key="refresh_message")
        setting.value = "updated value"
        appconfig_client.set_configuration_setting(setting)
        static_setting = appconfig_client.get_configuration_setting(key="non_refreshed_message")
        static_setting.value = "updated static"
        appconfig_client.set_configuration_setting(static_setting)

        # Waiting for the refresh interval to pass
        time.sleep(2)

        client.refresh()
        assert client["refresh_message"] == "original value"
        assert client["non_refreshed_message"] == "Static"
        assert mock_callback.call_count == 0

        setting.value = "original value"
        appconfig_client.set_configuration_setting(setting)
        static_setting.value = "Static"
        appconfig_client.set_configuration_setting(static_setting)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.web import WebSiteManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestWebSiteManagementRecommendationsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(WebSiteManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_list(self, resource_group):
        response = self.client.recommendations.list(
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_reset_all_filters(self, resource_group):
        response = self.client.recommendations.reset_all_filters(
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_disable_recommendation_for_subscription(self, resource_group):
        response = self.client.recommendations.disable_recommendation_for_subscription(
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_list_history_for_hosting_environment(self, resource_group):
        response = self.client.recommendations.list_history_for_hosting_environment(
            resource_group_name=resource_group.name,
            hosting_environment_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_list_recommended_rules_for_hosting_environment(self, resource_group):
        response = self.client.recommendations.list_recommended_rules_for_hosting_environment(
            resource_group_name=resource_group.name,
            hosting_environment_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_disable_all_for_hosting_environment(self, resource_group):
        response = self.client.recommendations.disable_all_for_hosting_environment(
            resource_group_name=resource_group.name,
            environment_name="str",
            hosting_environment_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_reset_all_filters_for_hosting_environment(self, resource_group):
        response = self.client.recommendations.reset_all_filters_for_hosting_environment(
            resource_group_name=resource_group.name,
            environment_name="str",
            hosting_environment_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_get_rule_details_by_hosting_environment(self, resource_group):
        response = self.client.recommendations.get_rule_details_by_hosting_environment(
            resource_group_name=resource_group.name,
            hosting_environment_name="str",
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_disable_recommendation_for_hosting_environment(self, resource_group):
        response = self.client.recommendations.disable_recommendation_for_hosting_environment(
            resource_group_name=resource_group.name,
            environment_name="str",
            name="str",
            hosting_environment_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_list_history_for_web_app(self, resource_group):
        response = self.client.recommendations.list_history_for_web_app(
            resource_group_name=resource_group.name,
            site_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_list_recommended_rules_for_web_app(self, resource_group):
        response = self.client.recommendations.list_recommended_rules_for_web_app(
            resource_group_name=resource_group.name,
            site_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_disable_all_for_web_app(self, resource_group):
        response = self.client.recommendations.disable_all_for_web_app(
            resource_group_name=resource_group.name,
            site_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_reset_all_filters_for_web_app(self, resource_group):
        response = self.client.recommendations.reset_all_filters_for_web_app(
            resource_group_name=resource_group.name,
            site_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_get_rule_details_by_web_app(self, resource_group):
        response = self.client.recommendations.get_rule_details_by_web_app(
            resource_group_name=resource_group.name,
            site_name="str",
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_recommendations_disable_recommendation_for_site(self, resource_group):
        response = self.client.recommendations.disable_recommendation_for_site(
            resource_group_name=resource_group.name,
            site_name="str",
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
DESCRIPTION:
    This sample demonstrates how to get a chat completions response from the service using a synchronous client,
    with Entra ID authentication.

PREREQUISITES:
    1) An AI model deployed to an "AI Services" resource, in your AI Foundry Project, using Entra ID authentication.
    2) You have the "Cognitive Services User" role assigned to you in the "AI Service" resource. Do that in your
       Azure portal (https://portal.azure.com/), under the "Access control (IAM)" tab of your "AI Service" resource.

USAGE:
    Set these two environment variables before running the sample:

    1) AZURE_AI_CHAT_ENDPOINT - Your endpoint URL for model inference, in the form 
       "https://<your-ai-services-resouce-name>.services.ai.azure.com/models".
       This URL is shown in the model card page in your AI Foundry project: Click on "Models + endpoints" tab,
       then click on the relevant model name to open the model card page. Copy the URL up to and including the
       "/models" route.
       
    2) AZURE_AI_CHAT_MODEL_DEPLOYMENT_NAME - Your model deployment name, as appears under the "Name"
       in the "Deployment info" section of the model card page. Note that this may be different than the
       "Model name" shown in the card.

    Open a console window and make sure you have latest version of "az" CLI installed.
    See https://learn.microsoft.com/cli/azure/install-azure-cli

    Run "az login" and pick the subscription/tenant that was used by your AI Foundry project.

    Now run the sample:
        python sample_chat_completions_with_entra_id_auth.py
"""


def sample_chat_completions_with_entra_id_auth():
    import os

    try:
        endpoint = os.environ["AZURE_AI_CHAT_ENDPOINT"]
        model_deployment_name = os.environ["AZURE_AI_CHAT_MODEL_DEPLOYMENT_NAME"]
    except KeyError:
        print("Missing environment variable 'AZURE_AI_CHAT_ENDPOINT' or 'AZURE_AI_CHAT_MODEL_DEPLOYMENT_NAME'")
        print("Set them before running this sample.")
        exit()

    from azure.ai.inference import ChatCompletionsClient
    from azure.ai.inference.models import SystemMessage, UserMessage
    from azure.identity import DefaultAzureCredential

    client = ChatCompletionsClient(
        endpoint=endpoint,
        credential=DefaultAzureCredential(),
        credential_scopes=["https://cognitiveservices.azure.com/.default"],
    )

    response = client.complete(
        messages=[
            SystemMessage("You are a helpful assistant."),
            UserMessage("How many feet are in a mile?"),
        ],
        model=model_deployment_name,
    )

    print(response.choices[0].message.content)


if __name__ == "__main__":
    sample_chat_completions_with_entra_id_auth()



import asyncio
import hashlib
import os
from typing import Dict, List, Union
from urllib.parse import urlparse

from azure.core.pipeline import policies
from devtools_testutils.aio import recorded_by_proxy_async
from devtools_testutils import (
    PemCertificate,
    create_combined_bundle,
    is_live,
    is_live_and_not_recording,
    recorded_by_proxy,
    set_function_recording_options,
)

from azure.confidentialledger.aio import ConfidentialLedgerClient
from azure.confidentialledger import ConfidentialLedgerCertificateCredential

from _shared.constants import (
    TEST_PROXY_CERT,
    USER_CERTIFICATE_THUMBPRINT,
    USER_CERTIFICATE_PRIVATE_KEY,
    USER_CERTIFICATE_PUBLIC_KEY,
)
from _shared.testcase import ConfidentialLedgerPreparer, ConfidentialLedgerTestCase


class TestConfidentialLedgerClient(ConfidentialLedgerTestCase):
    async def create_confidentialledger_client(
        self, endpoint, ledger_id, use_aad_auth,
    ):
        # Always explicitly fetch the TLS certificate.
        network_cert = await self.set_ledger_identity_async(ledger_id)

        # The ACL instance should already have the potential AAD user added as an Administrator.
        credential = self.get_credential(ConfidentialLedgerClient, is_async=True)
        aad_based_client = self.create_client_from_credential(
            ConfidentialLedgerClient,
            credential=credential,
            endpoint=endpoint,
            ledger_certificate_path=self.network_certificate_path,  # type: ignore
        )

        certificate_credential = ConfidentialLedgerCertificateCredential(
            certificate_path=self.user_certificate_path
        )
        certificate_based_client = ConfidentialLedgerClient(
            credential=certificate_credential,
            endpoint=endpoint,
            ledger_certificate_path=self.network_certificate_path,  # type: ignore
        )

        # The Confidential Ledger always presents a self-signed certificate, so add that certificate
        # to the recording options for the Confidential Ledger endpoint.
        function_recording_options: Dict[str, Union[str, List[PemCertificate]]] = {
            "tls_certificate": network_cert,
            "tls_certificate_host": urlparse(endpoint).netloc,
        }
        if is_live():
            set_function_recording_options(**function_recording_options)

        if not is_live_and_not_recording():
            # For live, non-recorded tests, we want to test normal client behavior so the only
            # certificate used for TLS verification is the Confidential Ledger identity certificate.
            #
            # However, in this case outbound connections are to the proxy, so the certificate used
            # for verifying the TLS connection should actually be the test proxy's certificate.
            # With that in mind, we'll update the file at self.network_certificate_path to be a
            # certificate bundle including both the ledger's TLS certificate (though technically
            # unnecessary I think) and the test-proxy certificate. This way the client setup (i.e.
            # the logic for overriding the default certificate verification) is still tested when
            # the test-proxy is involved.
            #
            # Note the combined bundle should be created *after* any os.remove calls so we don't 
            # interfere with auto-magic certificate retrieval tests.
            create_combined_bundle(
                [self.network_certificate_path, TEST_PROXY_CERT],
                self.network_certificate_path
            )

        if not use_aad_auth:
            # We need to add the certificate-based user as an Administrator.
            try: 
                await aad_based_client.create_or_update_ledger_user(
                    USER_CERTIFICATE_THUMBPRINT, {"assignedRoles": ["Administrator"]}
                )
            finally:
                await aad_based_client.close()

            # Sleep to make sure all replicas know the user is added.
            await asyncio.sleep(3)

            # Update the options to account for certificate-based authentication.
            function_recording_options["certificates"] = [
                PemCertificate(key=USER_CERTIFICATE_PRIVATE_KEY, data=USER_CERTIFICATE_PUBLIC_KEY)
            ]
            if is_live():
                set_function_recording_options(**function_recording_options)

            client = certificate_based_client
        else:
            client = aad_based_client

        return client

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_append_entry_flow_aad_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=True
        )
        try:
            await self.append_entry_flow_actions(client)
        finally:
            await client.close()

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_append_entry_flow_cert_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=False
        )
        try:
            await self.append_entry_flow_actions(client)
        finally:
            await client.close()

    async def append_entry_flow_actions(self, client):
        entry_contents = "Test entry from Python SDK"
        append_result = await client.create_ledger_entry({"contents": entry_contents})
        assert append_result["transactionId"]
        assert append_result["collectionId"]

        append_result_sub_ledger_id = append_result["collectionId"]
        append_result_transaction_id = append_result["transactionId"]

        poller = await client.begin_wait_for_commit(
            transaction_id=append_result_transaction_id
        )
        await poller.wait()

        transaction_status = await client.get_transaction_status(
            transaction_id=append_result_transaction_id
        )
        assert transaction_status["transactionId"] == append_result_transaction_id
        assert transaction_status["state"] == "Committed"

        poller = await client.begin_get_receipt(
            transaction_id=append_result_transaction_id
        )
        receipt = await poller.result()
        assert receipt["transactionId"] == append_result_transaction_id
        assert receipt["receipt"]

        latest_entry = await client.get_current_ledger_entry()
        # The transaction ids may not be equal in the unfortunate edge case where an internal
        # operation occurs after the ledger append (e.g. because a node was restarted). Then,
        # the latest id will be higher.
        latest_entry_view = int(latest_entry["transactionId"].split(".")[0])
        latest_entry_seqno = int(latest_entry["transactionId"].split(".")[-1])
        append_result_view = int(append_result_transaction_id.split(".")[0])
        append_result_seqno = int(append_result_transaction_id.split(".")[-1])
        assert latest_entry_view >= append_result_view and latest_entry_seqno >= append_result_seqno
        assert latest_entry["contents"] == entry_contents
        assert latest_entry["collectionId"] == append_result_sub_ledger_id

        poller = await client.begin_create_ledger_entry(
            {"contents": "Test entry 2 from Python SDK"}
        )
        await poller.wait()

        latest_entry = await client.get_current_ledger_entry()
        assert latest_entry["transactionId"] != append_result_transaction_id
        assert latest_entry["contents"] != entry_contents
        assert latest_entry["collectionId"] == append_result_sub_ledger_id

        poller = await client.begin_get_ledger_entry(
            transaction_id=append_result_transaction_id
        )
        original_entry = await poller.result()
        assert original_entry["entry"]["transactionId"] == append_result_transaction_id
        assert original_entry["entry"]["contents"] == entry_contents
        assert original_entry["entry"]["collectionId"] == append_result_sub_ledger_id

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_append_entry_flow_with_collection_id_aad_user(
        self, **kwargs,
    ):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=True
        )
        try:
            await self.append_entry_flow_with_collection_id_actions(client)
        finally:
            await client.close()

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_append_entry_flow_with_collection_id_cert_user(
        self, **kwargs,
    ):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=False
        )
        try:
            await self.append_entry_flow_with_collection_id_actions(client)
        finally:
            await client.close()

    async def append_entry_flow_with_collection_id_actions(self, client):
        collection_id = "132"
        entry_contents = f"Test entry from Python SDK. Collection: {collection_id}"
        append_result = await client.create_ledger_entry(
            {"contents": entry_contents},
            collection_id=collection_id,
        )
        assert append_result["transactionId"]
        assert append_result["collectionId"] == collection_id

        append_result_sub_ledger_id = append_result["collectionId"]
        append_result_transaction_id = append_result["transactionId"]

        poller = await client.begin_wait_for_commit(
            transaction_id=append_result_transaction_id
        )
        await poller.wait()

        transaction_status = await client.get_transaction_status(
            transaction_id=append_result_transaction_id
        )
        assert transaction_status
        assert transaction_status["state"] == "Committed"

        poller = await client.begin_get_receipt(
            transaction_id=append_result_transaction_id
        )
        receipt = await poller.result()
        assert receipt["transactionId"] == append_result_transaction_id
        assert receipt["receipt"]

        latest_entry = await client.get_current_ledger_entry(
            collection_id=collection_id
        )
        # The transaction ids may not be equal in the unfortunate edge case where an internal
        # operation occurs after the ledger append (e.g. because a node was restarted). Then,
        # the latest id will be higher.
        latest_entry_view = int(latest_entry["transactionId"].split(".")[0])
        latest_entry_seqno = int(latest_entry["transactionId"].split(".")[-1])
        append_result_view = int(append_result_transaction_id.split(".")[0])
        append_result_seqno = int(append_result_transaction_id.split(".")[-1])
        assert latest_entry_view >= append_result_view and latest_entry_seqno >= append_result_seqno
        assert latest_entry["contents"] == entry_contents
        assert latest_entry["collectionId"] == append_result_sub_ledger_id

        poller = await client.begin_create_ledger_entry(
            {"contents": f"Test entry 2 from Python SDK. Collection: {collection_id}"},
            collection_id=collection_id,
        )
        await poller.wait()

        latest_entry = await client.get_current_ledger_entry(
            collection_id=collection_id
        )
        assert latest_entry["transactionId"] != append_result_transaction_id
        assert latest_entry["contents"] != entry_contents
        assert latest_entry["collectionId"] == collection_id

        poller = await client.begin_get_ledger_entry(
            transaction_id=append_result_transaction_id,
            collection_id=collection_id,
        )
        original_entry = await poller.result()
        assert original_entry["entry"]["transactionId"] == append_result_transaction_id
        assert original_entry["entry"]["contents"] == entry_contents
        assert original_entry["entry"]["collectionId"] == append_result_sub_ledger_id

        collections = client.list_collections()
        collection_ids = set()
        async for collection in collections:
            collection_ids.add(collection["collectionId"])
        assert collection_id in collection_ids

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_range_query_aad_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=True
        )
        try:
            await self.range_query_actions(client)
        finally:
            await client.close()

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_range_query_cert_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=False
        )
        try:
            await self.range_query_actions(client)
        finally:
            await client.close()

    async def range_query_actions(self, client):
        num_collections = 5

        # Send 2 messages per collection. No need to test pagination over many pages because:
        # 1. Each page is 1000 entries, so we'd have to send 1001 * num_collections messages and
        #    and read them all; not ideal!
        # 2. The pagination _should_ be tested often enough because the first response is typically
        #    not ready, with a nextLink pointing at the original query and an empty list of entires,
        #    as the historical query is just starting.
        num_messages_sent = num_collections * 2

        messages = {m: [] for m in range(num_collections)}
        for i in range(num_messages_sent):
            message = "message-{0}".format(i)
            kwargs = (
                {} if num_collections == 0 else {"collection_id": "{0}".format(i % num_collections)}
            )

            if i != num_messages_sent - 1:
                append_result = await client.create_ledger_entry(
                    {"contents": message}, **kwargs
                )
            else:
                append_poller = await client.begin_create_ledger_entry(
                    {"contents": message},
                    **kwargs
                )
                append_result = await append_poller.result()

            messages[i % num_collections].append(
                (append_result["transactionId"], message, kwargs)
            )

        num_matched = 0
        for i in range(num_collections):
            query_result = client.list_ledger_entries(
                from_transaction_id=messages[i][0][0],
                to_transaction_id=messages[i][-1][0],
                **messages[i][0][2],
            )
            index = 0
            async for historical_entry in query_result:
                assert historical_entry["transactionId"] == messages[i][index][0]
                assert historical_entry["contents"] == messages[i][index][1]
                collection_id = messages[i][index][2].get("collection_id", None)
                if collection_id is not None:
                    assert historical_entry["collectionId"] == collection_id

                index += 1
                num_matched += 1

        assert num_matched == num_messages_sent

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_user_management_aad_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=True
        )
        try:
            await self.user_management_actions(client)
        finally:
            await client.close()

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_user_management_cert_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=False
        )
        try:
            await self.user_management_actions(client)
        finally:
            await client.close()

    async def user_management_actions(self, client):
        aad_user_id = "0" * 36  # AAD Object Ids have length 36
        cert_user_id = (
            "7F:75:58:60:70:A8:B6:15:A2:CD:24:55:25:B9:64:49:F8:BF:F0:E3:4D:92:EA:B2:8C:30:E6:2D:F4"
            ":77:30:1F"
        )
        for user_id in [aad_user_id, cert_user_id]:
            await client.delete_ledger_user(user_id)

            await asyncio.sleep(3)  # Let the DELETE user operation be committed, just in case.

            user = await client.create_or_update_ledger_user(user_id, {"assignedRoles": ["Contributor"]})
            assert user["userId"] == user_id
            assert user["assignedRoles"] == ["Contributor"]

            await asyncio.sleep(3)  # Let the PATCH user operation be committed, just in case.

            user = await client.get_ledger_user(user_id)
            assert user["userId"] == user_id
            assert user["assignedRoles"] == ["Contributor"]

            user = await client.create_or_update_ledger_user(user_id, {"assignedRoles": ["Reader"]})
            assert user["userId"] == user_id
            assert user["assignedRoles"] == ["Reader"]

            await asyncio.sleep(3)  # Let the PATCH user operation be committed, just in case.

            user = await client.get_ledger_user(user_id)
            assert user["userId"] == user_id
            assert user["assignedRoles"] == ["Contributor","Reader"]

            await client.delete_ledger_user(user_id)
            await asyncio.sleep(3)  # Let the DELETE user operation be committed, just in case.

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_verification_methods_aad_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=True
        )
        try:
            await self.verification_methods_actions(client)
        finally:
            await client.close()

    @ConfidentialLedgerPreparer()
    @recorded_by_proxy_async
    async def test_verification_methods_cert_user(self, **kwargs):
        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")
        client = await self.create_confidentialledger_client(
            confidentialledger_endpoint, confidentialledger_id, use_aad_auth=False
        )
        try:
            await self.verification_methods_actions(client)
        finally:
            await client.close()

    async def verification_methods_actions(self, client):
        consortium = client.list_consortium_members()
        consortium_size = 0
        async for member in consortium:
            assert member["certificate"]
            assert member["id"]
            consortium_size += 1
        assert consortium_size == 1

        constitution = await client.get_constitution()
        assert constitution["script"]
        assert constitution["digest"]
        assert (
            constitution["digest"].lower() == 
            hashlib.sha256(constitution["script"].encode()).hexdigest().lower()
        )

        ledger_enclaves = await client.get_enclave_quotes()
        assert len(ledger_enclaves["enclaveQuotes"]) == 3
        assert ledger_enclaves["currentNodeId"] in ledger_enclaves["enclaveQuotes"]
        for node_id, quote in ledger_enclaves["enclaveQuotes"].items():
            assert node_id == quote["nodeId"]
            assert quote["nodeId"]
            assert quote["raw"]
            assert quote["quoteVersion"]

    # Must use recorded_by_proxy (non-async) because the constructor makes a synchronous call that
    # will be ignored by the proxy when running in recorded mode.
    @ConfidentialLedgerPreparer()
    @recorded_by_proxy
    def test_tls_cert_convenience_aad_user(self, **kwargs):
        try:
            os.remove(self.network_certificate_path)  # Remove file so the auto-magic kicks in.
        except FileNotFoundError:
            pass

        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")

        # Create the client directly instead of going through the create_confidentialledger_client
        # as we don't need any additional setup.
        credential = self.get_credential(ConfidentialLedgerClient, is_async=False)
        self.create_client_from_credential(
            ConfidentialLedgerClient,
            credential=credential,
            endpoint=confidentialledger_endpoint,
            ledger_certificate_path=self.network_certificate_path,  # type: ignore
            # Provide an authentication policy so that the default AsyncBearerTokenCredentialPolicy
            # is not used. Using that instead can cause an error as there is no event loop running
            # for this non-async test.
            authentication_policy=policies.BearerTokenCredentialPolicy(
                credential, 
                *["https://confidential-ledger.azure.com/.default"], 
                **kwargs,
            ),
        )

        self.tls_cert_convenience_actions(confidentialledger_id)

    # Must use recorded_by_proxy (non-async) because the constructor makes a synchronous call that
    # will be ignored by the proxy when running in recorded mode.
    @ConfidentialLedgerPreparer()
    @recorded_by_proxy
    def test_tls_cert_convenience_cert_user(self, **kwargs):
        try:
            os.remove(self.network_certificate_path)  # Remove file so the auto-magic kicks in.
        except FileNotFoundError:
            pass

        confidentialledger_endpoint = kwargs.pop("confidentialledger_endpoint")
        confidentialledger_id = kwargs.pop("confidentialledger_id")

        # Create the client directly instead of going through the create_confidentialledger_client
        # as we don't need any additional setup.
        certificate_credential = ConfidentialLedgerCertificateCredential(
            certificate_path=self.user_certificate_path
        )
        ConfidentialLedgerClient(
            credential=certificate_credential,
            endpoint=confidentialledger_endpoint,
            ledger_certificate_path=self.network_certificate_path,  # type: ignore
        )

        self.tls_cert_convenience_actions(confidentialledger_id)

    def tls_cert_convenience_actions(self, confidentialledger_id: str):
        with open(self.network_certificate_path) as infile:
            certificate = infile.read()

        expected_cert = self.set_ledger_identity(confidentialledger_id)

        assert certificate == expected_cert



# coding=utf-8
# pylint: disable=too-many-lines
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator (autorest: 3.9.7, generator: @autorest/python@6.7.8)
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
import sys
from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union

from .. import _serialization

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from .. import models as _models
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object


class AcrAccessToken(_serialization.Model):
    """AcrAccessToken.

    :ivar access_token: The access token for performing authenticated requests.
    :vartype access_token: str
    """

    _attribute_map = {
        "access_token": {"key": "access_token", "type": "str"},
    }

    def __init__(self, *, access_token: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword access_token: The access token for performing authenticated requests.
        :paramtype access_token: str
        """
        super().__init__(**kwargs)
        self.access_token = access_token


class AcrErrorInfo(_serialization.Model):
    """Error information.

    :ivar code: Error code.
    :vartype code: str
    :ivar message: Error message.
    :vartype message: str
    :ivar detail: Error details.
    :vartype detail: JSON
    """

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "detail": {"key": "detail", "type": "object"},
    }

    def __init__(
        self, *, code: Optional[str] = None, message: Optional[str] = None, detail: Optional[JSON] = None, **kwargs: Any
    ) -> None:
        """
        :keyword code: Error code.
        :paramtype code: str
        :keyword message: Error message.
        :paramtype message: str
        :keyword detail: Error details.
        :paramtype detail: JSON
        """
        super().__init__(**kwargs)
        self.code = code
        self.message = message
        self.detail = detail


class AcrErrors(_serialization.Model):
    """Acr error response describing why the operation failed.

    :ivar errors: Array of detailed error.
    :vartype errors: list[~container_registry.models.AcrErrorInfo]
    """

    _attribute_map = {
        "errors": {"key": "errors", "type": "[AcrErrorInfo]"},
    }

    def __init__(self, *, errors: Optional[List["_models.AcrErrorInfo"]] = None, **kwargs: Any) -> None:
        """
        :keyword errors: Array of detailed error.
        :paramtype errors: list[~container_registry.models.AcrErrorInfo]
        """
        super().__init__(**kwargs)
        self.errors = errors


class AcrManifests(_serialization.Model):
    """Manifest attributes.

    :ivar registry_login_server: Registry login server name. This is likely to be similar to
     {registry-name}.azurecr.io.
    :vartype registry_login_server: str
    :ivar repository: Image name.
    :vartype repository: str
    :ivar manifests: List of manifests.
    :vartype manifests: list[~container_registry.models.ManifestAttributesBase]
    :ivar link:
    :vartype link: str
    """

    _attribute_map = {
        "registry_login_server": {"key": "registry", "type": "str"},
        "repository": {"key": "imageName", "type": "str"},
        "manifests": {"key": "manifests", "type": "[ManifestAttributesBase]"},
        "link": {"key": "link", "type": "str"},
    }

    def __init__(
        self,
        *,
        registry_login_server: Optional[str] = None,
        repository: Optional[str] = None,
        manifests: Optional[List["_models.ManifestAttributesBase"]] = None,
        link: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword registry_login_server: Registry login server name. This is likely to be similar to
         {registry-name}.azurecr.io.
        :paramtype registry_login_server: str
        :keyword repository: Image name.
        :paramtype repository: str
        :keyword manifests: List of manifests.
        :paramtype manifests: list[~container_registry.models.ManifestAttributesBase]
        :keyword link:
        :paramtype link: str
        """
        super().__init__(**kwargs)
        self.registry_login_server = registry_login_server
        self.repository = repository
        self.manifests = manifests
        self.link = link


class AcrRefreshToken(_serialization.Model):
    """AcrRefreshToken.

    :ivar refresh_token: The refresh token to be used for generating access tokens.
    :vartype refresh_token: str
    """

    _attribute_map = {
        "refresh_token": {"key": "refresh_token", "type": "str"},
    }

    def __init__(self, *, refresh_token: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword refresh_token: The refresh token to be used for generating access tokens.
        :paramtype refresh_token: str
        """
        super().__init__(**kwargs)
        self.refresh_token = refresh_token


class Annotations(_serialization.Model):  # pylint: disable=too-many-instance-attributes
    """Additional information provided through arbitrary metadata.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, JSON]
    :ivar created: Date and time on which the image was built (string, date-time as defined by
     https://tools.ietf.org/html/rfc3339#section-5.6).
    :vartype created: ~datetime.datetime
    :ivar authors: Contact details of the people or organization responsible for the image.
    :vartype authors: str
    :ivar url: URL to find more information on the image.
    :vartype url: str
    :ivar documentation: URL to get documentation on the image.
    :vartype documentation: str
    :ivar source: URL to get source code for building the image.
    :vartype source: str
    :ivar version: Version of the packaged software. The version MAY match a label or tag in the
     source code repository, may also be Semantic versioning-compatible.
    :vartype version: str
    :ivar revision: Source control revision identifier for the packaged software.
    :vartype revision: str
    :ivar vendor: Name of the distributing entity, organization or individual.
    :vartype vendor: str
    :ivar licenses: License(s) under which contained software is distributed as an SPDX License
     Expression.
    :vartype licenses: str
    :ivar name: Name of the reference for a target.
    :vartype name: str
    :ivar title: Human-readable title of the image.
    :vartype title: str
    :ivar description: Human-readable description of the software packaged in the image.
    :vartype description: str
    """

    _attribute_map = {
        "additional_properties": {"key": "", "type": "{object}"},
        "created": {"key": "org\\.opencontainers\\.image\\.created", "type": "iso-8601"},
        "authors": {"key": "org\\.opencontainers\\.image\\.authors", "type": "str"},
        "url": {"key": "org\\.opencontainers\\.image\\.url", "type": "str"},
        "documentation": {"key": "org\\.opencontainers\\.image\\.documentation", "type": "str"},
        "source": {"key": "org\\.opencontainers\\.image\\.source", "type": "str"},
        "version": {"key": "org\\.opencontainers\\.image\\.version", "type": "str"},
        "revision": {"key": "org\\.opencontainers\\.image\\.revision", "type": "str"},
        "vendor": {"key": "org\\.opencontainers\\.image\\.vendor", "type": "str"},
        "licenses": {"key": "org\\.opencontainers\\.image\\.licenses", "type": "str"},
        "name": {"key": "org\\.opencontainers\\.image\\.ref\\.name", "type": "str"},
        "title": {"key": "org\\.opencontainers\\.image\\.title", "type": "str"},
        "description": {"key": "org\\.opencontainers\\.image\\.description", "type": "str"},
    }

    def __init__(
        self,
        *,
        additional_properties: Optional[Dict[str, JSON]] = None,
        created: Optional[datetime.datetime] = None,
        authors: Optional[str] = None,
        url: Optional[str] = None,
        documentation: Optional[str] = None,
        source: Optional[str] = None,
        version: Optional[str] = None,
        revision: Optional[str] = None,
        vendor: Optional[str] = None,
        licenses: Optional[str] = None,
        name: Optional[str] = None,
        title: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, JSON]
        :keyword created: Date and time on which the image was built (string, date-time as defined by
         https://tools.ietf.org/html/rfc3339#section-5.6).
        :paramtype created: ~datetime.datetime
        :keyword authors: Contact details of the people or organization responsible for the image.
        :paramtype authors: str
        :keyword url: URL to find more information on the image.
        :paramtype url: str
        :keyword documentation: URL to get documentation on the image.
        :paramtype documentation: str
        :keyword source: URL to get source code for building the image.
        :paramtype source: str
        :keyword version: Version of the packaged software. The version MAY match a label or tag in the
         source code repository, may also be Semantic versioning-compatible.
        :paramtype version: str
        :keyword revision: Source control revision identifier for the packaged software.
        :paramtype revision: str
        :keyword vendor: Name of the distributing entity, organization or individual.
        :paramtype vendor: str
        :keyword licenses: License(s) under which contained software is distributed as an SPDX License
         Expression.
        :paramtype licenses: str
        :keyword name: Name of the reference for a target.
        :paramtype name: str
        :keyword title: Human-readable title of the image.
        :paramtype title: str
        :keyword description: Human-readable description of the software packaged in the image.
        :paramtype description: str
        """
        super().__init__(**kwargs)
        self.additional_properties = additional_properties
        self.created = created
        self.authors = authors
        self.url = url
        self.documentation = documentation
        self.source = source
        self.version = version
        self.revision = revision
        self.vendor = vendor
        self.licenses = licenses
        self.name = name
        self.title = title
        self.description = description


class ArtifactManifestPlatform(_serialization.Model):
    """The artifact's platform, consisting of operating system and architecture.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar digest: Manifest digest. Required.
    :vartype digest: str
    :ivar architecture: CPU architecture. Known values are: "386", "amd64", "arm", "arm64", "mips",
     "mipsle", "mips64", "mips64le", "ppc64", "ppc64le", "riscv64", "s390x", and "wasm".
    :vartype architecture: str or ~container_registry.models.ArtifactArchitecture
    :ivar operating_system: Operating system. Known values are: "aix", "android", "darwin",
     "dragonfly", "freebsd", "illumos", "ios", "js", "linux", "netbsd", "openbsd", "plan9",
     "solaris", and "windows".
    :vartype operating_system: str or ~container_registry.models.ArtifactOperatingSystem
    """

    _validation = {
        "digest": {"required": True, "readonly": True},
        "architecture": {"readonly": True},
        "operating_system": {"readonly": True},
    }

    _attribute_map = {
        "digest": {"key": "digest", "type": "str"},
        "architecture": {"key": "architecture", "type": "str"},
        "operating_system": {"key": "os", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.digest = None
        self.architecture = None
        self.operating_system = None


class ArtifactManifestProperties(_serialization.Model):
    """Manifest attributes details.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar registry_login_server: Registry login server name. This is likely to be similar to
     {registry-name}.azurecr.io.
    :vartype registry_login_server: str
    :ivar repository_name: Repository name.
    :vartype repository_name: str
    :ivar manifest: Manifest attributes. Required.
    :vartype manifest: ~container_registry.models.ManifestAttributesBase
    """

    _validation = {
        "registry_login_server": {"readonly": True},
        "repository_name": {"readonly": True},
        "manifest": {"required": True, "readonly": True},
    }

    _attribute_map = {
        "registry_login_server": {"key": "registry", "type": "str"},
        "repository_name": {"key": "imageName", "type": "str"},
        "manifest": {"key": "manifest", "type": "ManifestAttributesBase"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.registry_login_server = None
        self.repository_name = None
        self.manifest = None


class ArtifactTagProperties(_serialization.Model):
    """Tag attributes.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar registry_login_server: Registry login server name. This is likely to be similar to
     {registry-name}.azurecr.io. Required.
    :vartype registry_login_server: str
    :ivar repository_name: Image name. Required.
    :vartype repository_name: str
    :ivar tag: List of tag attribute details. Required.
    :vartype tag: ~container_registry.models.TagAttributesBase
    """

    _validation = {
        "registry_login_server": {"required": True, "readonly": True},
        "repository_name": {"required": True, "readonly": True},
        "tag": {"required": True, "readonly": True},
    }

    _attribute_map = {
        "registry_login_server": {"key": "registry", "type": "str"},
        "repository_name": {"key": "imageName", "type": "str"},
        "tag": {"key": "tag", "type": "TagAttributesBase"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.registry_login_server = None
        self.repository_name = None
        self.tag = None


class ContainerRepositoryProperties(_serialization.Model):
    """Properties of this repository.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar registry_login_server: Registry login server name. This is likely to be similar to
     {registry-name}.azurecr.io. Required.
    :vartype registry_login_server: str
    :ivar name: Image name. Required.
    :vartype name: str
    :ivar created_on: Image created time. Required.
    :vartype created_on: ~datetime.datetime
    :ivar last_updated_on: Image last update time. Required.
    :vartype last_updated_on: ~datetime.datetime
    :ivar manifest_count: Number of the manifests. Required.
    :vartype manifest_count: int
    :ivar tag_count: Number of the tags. Required.
    :vartype tag_count: int
    :ivar changeable_attributes: Writeable properties of the resource. Required.
    :vartype changeable_attributes: ~container_registry.models.RepositoryWriteableProperties
    """

    _validation = {
        "registry_login_server": {"required": True, "readonly": True},
        "name": {"required": True, "readonly": True},
        "created_on": {"required": True, "readonly": True},
        "last_updated_on": {"required": True, "readonly": True},
        "manifest_count": {"required": True, "readonly": True},
        "tag_count": {"required": True, "readonly": True},
        "changeable_attributes": {"required": True},
    }

    _attribute_map = {
        "registry_login_server": {"key": "registry", "type": "str"},
        "name": {"key": "imageName", "type": "str"},
        "created_on": {"key": "createdTime", "type": "iso-8601"},
        "last_updated_on": {"key": "lastUpdateTime", "type": "iso-8601"},
        "manifest_count": {"key": "manifestCount", "type": "int"},
        "tag_count": {"key": "tagCount", "type": "int"},
        "changeable_attributes": {"key": "changeableAttributes", "type": "RepositoryWriteableProperties"},
    }

    def __init__(self, *, changeable_attributes: "_models.RepositoryWriteableProperties", **kwargs: Any) -> None:
        """
        :keyword changeable_attributes: Writeable properties of the resource. Required.
        :paramtype changeable_attributes: ~container_registry.models.RepositoryWriteableProperties
        """
        super().__init__(**kwargs)
        self.registry_login_server = None
        self.name = None
        self.created_on = None
        self.last_updated_on = None
        self.manifest_count = None
        self.tag_count = None
        self.changeable_attributes = changeable_attributes


class DeleteRepositoryResult(_serialization.Model):
    """Deleted repository.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar deleted_manifests: SHA of the deleted image.
    :vartype deleted_manifests: list[str]
    :ivar deleted_tags: Tag of the deleted image.
    :vartype deleted_tags: list[str]
    """

    _validation = {
        "deleted_manifests": {"readonly": True},
        "deleted_tags": {"readonly": True},
    }

    _attribute_map = {
        "deleted_manifests": {"key": "manifestsDeleted", "type": "[str]"},
        "deleted_tags": {"key": "tagsDeleted", "type": "[str]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.deleted_manifests = None
        self.deleted_tags = None


class Descriptor(_serialization.Model):
    """Docker V2 image layer descriptor including config and layers.

    :ivar media_type: Layer media type.
    :vartype media_type: str
    :ivar size: Layer size.
    :vartype size: int
    :ivar digest: Layer digest.
    :vartype digest: str
    :ivar urls: Specifies a list of URIs from which this object may be downloaded.
    :vartype urls: list[str]
    :ivar annotations: Additional information provided through arbitrary metadata.
    :vartype annotations: ~container_registry.models.Annotations
    """

    _attribute_map = {
        "media_type": {"key": "mediaType", "type": "str"},
        "size": {"key": "size", "type": "int"},
        "digest": {"key": "digest", "type": "str"},
        "urls": {"key": "urls", "type": "[str]"},
        "annotations": {"key": "annotations", "type": "Annotations"},
    }

    def __init__(
        self,
        *,
        media_type: Optional[str] = None,
        size: Optional[int] = None,
        digest: Optional[str] = None,
        urls: Optional[List[str]] = None,
        annotations: Optional["_models.Annotations"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword media_type: Layer media type.
        :paramtype media_type: str
        :keyword size: Layer size.
        :paramtype size: int
        :keyword digest: Layer digest.
        :paramtype digest: str
        :keyword urls: Specifies a list of URIs from which this object may be downloaded.
        :paramtype urls: list[str]
        :keyword annotations: Additional information provided through arbitrary metadata.
        :paramtype annotations: ~container_registry.models.Annotations
        """
        super().__init__(**kwargs)
        self.media_type = media_type
        self.size = size
        self.digest = digest
        self.urls = urls
        self.annotations = annotations


class FsLayer(_serialization.Model):
    """Image layer information.

    :ivar blob_sum: SHA of an image layer.
    :vartype blob_sum: str
    """

    _attribute_map = {
        "blob_sum": {"key": "blobSum", "type": "str"},
    }

    def __init__(self, *, blob_sum: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword blob_sum: SHA of an image layer.
        :paramtype blob_sum: str
        """
        super().__init__(**kwargs)
        self.blob_sum = blob_sum


class History(_serialization.Model):
    """A list of unstructured historical data for v1 compatibility.

    :ivar v1_compatibility: The raw v1 compatibility information.
    :vartype v1_compatibility: str
    """

    _attribute_map = {
        "v1_compatibility": {"key": "v1Compatibility", "type": "str"},
    }

    def __init__(self, *, v1_compatibility: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword v1_compatibility: The raw v1 compatibility information.
        :paramtype v1_compatibility: str
        """
        super().__init__(**kwargs)
        self.v1_compatibility = v1_compatibility


class ImageSignature(_serialization.Model):
    """Signature of a signed manifest.

    :ivar header: A JSON web signature.
    :vartype header: ~container_registry.models.JWK
    :ivar signature: A signature for the image manifest, signed by a libtrust private key.
    :vartype signature: str
    :ivar protected: The signed protected header.
    :vartype protected: str
    """

    _attribute_map = {
        "header": {"key": "header", "type": "JWK"},
        "signature": {"key": "signature", "type": "str"},
        "protected": {"key": "protected", "type": "str"},
    }

    def __init__(
        self,
        *,
        header: Optional["_models.JWK"] = None,
        signature: Optional[str] = None,
        protected: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword header: A JSON web signature.
        :paramtype header: ~container_registry.models.JWK
        :keyword signature: A signature for the image manifest, signed by a libtrust private key.
        :paramtype signature: str
        :keyword protected: The signed protected header.
        :paramtype protected: str
        """
        super().__init__(**kwargs)
        self.header = header
        self.signature = signature
        self.protected = protected


class JWK(_serialization.Model):
    """A JSON web signature.

    :ivar jwk: JSON web key parameter.
    :vartype jwk: ~container_registry.models.JWKHeader
    :ivar alg: The algorithm used to sign or encrypt the JWT.
    :vartype alg: str
    """

    _attribute_map = {
        "jwk": {"key": "jwk", "type": "JWKHeader"},
        "alg": {"key": "alg", "type": "str"},
    }

    def __init__(self, *, jwk: Optional["_models.JWKHeader"] = None, alg: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword jwk: JSON web key parameter.
        :paramtype jwk: ~container_registry.models.JWKHeader
        :keyword alg: The algorithm used to sign or encrypt the JWT.
        :paramtype alg: str
        """
        super().__init__(**kwargs)
        self.jwk = jwk
        self.alg = alg


class JWKHeader(_serialization.Model):
    """JSON web key parameter.

    :ivar crv: crv value.
    :vartype crv: str
    :ivar kid: kid value.
    :vartype kid: str
    :ivar kty: kty value.
    :vartype kty: str
    :ivar x: x value.
    :vartype x: str
    :ivar y: y value.
    :vartype y: str
    """

    _attribute_map = {
        "crv": {"key": "crv", "type": "str"},
        "kid": {"key": "kid", "type": "str"},
        "kty": {"key": "kty", "type": "str"},
        "x": {"key": "x", "type": "str"},
        "y": {"key": "y", "type": "str"},
    }

    def __init__(
        self,
        *,
        crv: Optional[str] = None,
        kid: Optional[str] = None,
        kty: Optional[str] = None,
        x: Optional[str] = None,
        y: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword crv: crv value.
        :paramtype crv: str
        :keyword kid: kid value.
        :paramtype kid: str
        :keyword kty: kty value.
        :paramtype kty: str
        :keyword x: x value.
        :paramtype x: str
        :keyword y: y value.
        :paramtype y: str
        """
        super().__init__(**kwargs)
        self.crv = crv
        self.kid = kid
        self.kty = kty
        self.x = x
        self.y = y


class Manifest(_serialization.Model):
    """Returns the requested manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
    }

    def __init__(self, *, schema_version: Optional[int] = None, **kwargs: Any) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        """
        super().__init__(**kwargs)
        self.schema_version = schema_version


class ManifestAttributesBase(_serialization.Model):
    """Manifest details.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar digest: Manifest. Required.
    :vartype digest: str
    :ivar size: Image size.
    :vartype size: int
    :ivar created_on: Created time. Required.
    :vartype created_on: ~datetime.datetime
    :ivar last_updated_on: Last update time. Required.
    :vartype last_updated_on: ~datetime.datetime
    :ivar architecture: CPU architecture. Known values are: "386", "amd64", "arm", "arm64", "mips",
     "mipsle", "mips64", "mips64le", "ppc64", "ppc64le", "riscv64", "s390x", and "wasm".
    :vartype architecture: str or ~container_registry.models.ArtifactArchitecture
    :ivar operating_system: Operating system. Known values are: "aix", "android", "darwin",
     "dragonfly", "freebsd", "illumos", "ios", "js", "linux", "netbsd", "openbsd", "plan9",
     "solaris", and "windows".
    :vartype operating_system: str or ~container_registry.models.ArtifactOperatingSystem
    :ivar related_artifacts: List of artifacts that are referenced by this manifest list, with
     information about the platform each supports.  This list will be empty if this is a leaf
     manifest and not a manifest list.
    :vartype related_artifacts: list[~container_registry.models.ArtifactManifestPlatform]
    :ivar tags: List of tags.
    :vartype tags: list[str]
    :ivar changeable_attributes: Writeable properties of the resource.
    :vartype changeable_attributes: ~container_registry.models.ManifestWriteableProperties
    """

    _validation = {
        "digest": {"required": True, "readonly": True},
        "size": {"readonly": True},
        "created_on": {"required": True, "readonly": True},
        "last_updated_on": {"required": True, "readonly": True},
        "architecture": {"readonly": True},
        "operating_system": {"readonly": True},
        "related_artifacts": {"readonly": True},
        "tags": {"readonly": True},
    }

    _attribute_map = {
        "digest": {"key": "digest", "type": "str"},
        "size": {"key": "imageSize", "type": "int"},
        "created_on": {"key": "createdTime", "type": "iso-8601"},
        "last_updated_on": {"key": "lastUpdateTime", "type": "iso-8601"},
        "architecture": {"key": "architecture", "type": "str"},
        "operating_system": {"key": "os", "type": "str"},
        "related_artifacts": {"key": "references", "type": "[ArtifactManifestPlatform]"},
        "tags": {"key": "tags", "type": "[str]"},
        "changeable_attributes": {"key": "changeableAttributes", "type": "ManifestWriteableProperties"},
    }

    def __init__(
        self, *, changeable_attributes: Optional["_models.ManifestWriteableProperties"] = None, **kwargs: Any
    ) -> None:
        """
        :keyword changeable_attributes: Writeable properties of the resource.
        :paramtype changeable_attributes: ~container_registry.models.ManifestWriteableProperties
        """
        super().__init__(**kwargs)
        self.digest = None
        self.size = None
        self.created_on = None
        self.last_updated_on = None
        self.architecture = None
        self.operating_system = None
        self.related_artifacts = None
        self.tags = None
        self.changeable_attributes = changeable_attributes


class ManifestAttributesManifest(_serialization.Model):
    """List of manifest attributes.

    :ivar references: List of manifest attributes details.
    :vartype references: list[~container_registry.models.ArtifactManifestPlatform]
    """

    _attribute_map = {
        "references": {"key": "references", "type": "[ArtifactManifestPlatform]"},
    }

    def __init__(self, *, references: Optional[List["_models.ArtifactManifestPlatform"]] = None, **kwargs: Any) -> None:
        """
        :keyword references: List of manifest attributes details.
        :paramtype references: list[~container_registry.models.ArtifactManifestPlatform]
        """
        super().__init__(**kwargs)
        self.references = references


class ManifestList(Manifest):
    """Returns the requested Docker multi-arch-manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar media_type: Media type for this Manifest.
    :vartype media_type: str
    :ivar manifests: List of V2 image layer information.
    :vartype manifests: list[~container_registry.models.ManifestListAttributes]
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "media_type": {"key": "mediaType", "type": "str"},
        "manifests": {"key": "manifests", "type": "[ManifestListAttributes]"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        media_type: Optional[str] = None,
        manifests: Optional[List["_models.ManifestListAttributes"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword media_type: Media type for this Manifest.
        :paramtype media_type: str
        :keyword manifests: List of V2 image layer information.
        :paramtype manifests: list[~container_registry.models.ManifestListAttributes]
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.media_type = media_type
        self.manifests = manifests


class ManifestListAttributes(_serialization.Model):
    """ManifestListAttributes.

    :ivar media_type: The MIME type of the referenced object. This will generally be
     application/vnd.docker.image.manifest.v2+json, but it could also be
     application/vnd.docker.image.manifest.v1+json.
    :vartype media_type: str
    :ivar size: The size in bytes of the object.
    :vartype size: int
    :ivar digest: The digest of the content, as defined by the Registry V2 HTTP API Specification.
    :vartype digest: str
    :ivar platform: The platform object describes the platform which the image in the manifest runs
     on. A full list of valid operating system and architecture values are listed in the Go language
     documentation for $GOOS and $GOARCH.
    :vartype platform: ~container_registry.models.Platform
    """

    _attribute_map = {
        "media_type": {"key": "mediaType", "type": "str"},
        "size": {"key": "size", "type": "int"},
        "digest": {"key": "digest", "type": "str"},
        "platform": {"key": "platform", "type": "Platform"},
    }

    def __init__(
        self,
        *,
        media_type: Optional[str] = None,
        size: Optional[int] = None,
        digest: Optional[str] = None,
        platform: Optional["_models.Platform"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword media_type: The MIME type of the referenced object. This will generally be
         application/vnd.docker.image.manifest.v2+json, but it could also be
         application/vnd.docker.image.manifest.v1+json.
        :paramtype media_type: str
        :keyword size: The size in bytes of the object.
        :paramtype size: int
        :keyword digest: The digest of the content, as defined by the Registry V2 HTTP API
         Specification.
        :paramtype digest: str
        :keyword platform: The platform object describes the platform which the image in the manifest
         runs on. A full list of valid operating system and architecture values are listed in the Go
         language documentation for $GOOS and $GOARCH.
        :paramtype platform: ~container_registry.models.Platform
        """
        super().__init__(**kwargs)
        self.media_type = media_type
        self.size = size
        self.digest = digest
        self.platform = platform


class ManifestWrapper(Manifest):  # pylint: disable=too-many-instance-attributes
    """Returns the requested manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar media_type: Media type for this Manifest.
    :vartype media_type: str
    :ivar manifests: (ManifestList, OCIIndex) List of V2 image layer information.
    :vartype manifests: list[~container_registry.models.ManifestListAttributes]
    :ivar config: (V2, OCI) Image config descriptor.
    :vartype config: ~container_registry.models.Descriptor
    :ivar layers: (V2, OCI) List of V2 image layer information.
    :vartype layers: list[~container_registry.models.Descriptor]
    :ivar annotations: (OCI, OCIIndex) Additional metadata.
    :vartype annotations: ~container_registry.models.Annotations
    :ivar architecture: (V1) CPU architecture.
    :vartype architecture: str
    :ivar name: (V1) Image name.
    :vartype name: str
    :ivar tag: (V1) Image tag.
    :vartype tag: str
    :ivar fs_layers: (V1) List of layer information.
    :vartype fs_layers: list[~container_registry.models.FsLayer]
    :ivar history: (V1) Image history.
    :vartype history: list[~container_registry.models.History]
    :ivar signatures: (V1) Image signature.
    :vartype signatures: list[~container_registry.models.ImageSignature]
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "media_type": {"key": "mediaType", "type": "str"},
        "manifests": {"key": "manifests", "type": "[ManifestListAttributes]"},
        "config": {"key": "config", "type": "Descriptor"},
        "layers": {"key": "layers", "type": "[Descriptor]"},
        "annotations": {"key": "annotations", "type": "Annotations"},
        "architecture": {"key": "architecture", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "tag": {"key": "tag", "type": "str"},
        "fs_layers": {"key": "fsLayers", "type": "[FsLayer]"},
        "history": {"key": "history", "type": "[History]"},
        "signatures": {"key": "signatures", "type": "[ImageSignature]"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        media_type: Optional[str] = None,
        manifests: Optional[List["_models.ManifestListAttributes"]] = None,
        config: Optional["_models.Descriptor"] = None,
        layers: Optional[List["_models.Descriptor"]] = None,
        annotations: Optional["_models.Annotations"] = None,
        architecture: Optional[str] = None,
        name: Optional[str] = None,
        tag: Optional[str] = None,
        fs_layers: Optional[List["_models.FsLayer"]] = None,
        history: Optional[List["_models.History"]] = None,
        signatures: Optional[List["_models.ImageSignature"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword media_type: Media type for this Manifest.
        :paramtype media_type: str
        :keyword manifests: (ManifestList, OCIIndex) List of V2 image layer information.
        :paramtype manifests: list[~container_registry.models.ManifestListAttributes]
        :keyword config: (V2, OCI) Image config descriptor.
        :paramtype config: ~container_registry.models.Descriptor
        :keyword layers: (V2, OCI) List of V2 image layer information.
        :paramtype layers: list[~container_registry.models.Descriptor]
        :keyword annotations: (OCI, OCIIndex) Additional metadata.
        :paramtype annotations: ~container_registry.models.Annotations
        :keyword architecture: (V1) CPU architecture.
        :paramtype architecture: str
        :keyword name: (V1) Image name.
        :paramtype name: str
        :keyword tag: (V1) Image tag.
        :paramtype tag: str
        :keyword fs_layers: (V1) List of layer information.
        :paramtype fs_layers: list[~container_registry.models.FsLayer]
        :keyword history: (V1) Image history.
        :paramtype history: list[~container_registry.models.History]
        :keyword signatures: (V1) Image signature.
        :paramtype signatures: list[~container_registry.models.ImageSignature]
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.media_type = media_type
        self.manifests = manifests
        self.config = config
        self.layers = layers
        self.annotations = annotations
        self.architecture = architecture
        self.name = name
        self.tag = tag
        self.fs_layers = fs_layers
        self.history = history
        self.signatures = signatures


class ManifestWriteableProperties(_serialization.Model):
    """Changeable attributes.

    :ivar can_delete: Delete enabled.
    :vartype can_delete: bool
    :ivar can_write: Write enabled.
    :vartype can_write: bool
    :ivar can_list: List enabled.
    :vartype can_list: bool
    :ivar can_read: Read enabled.
    :vartype can_read: bool
    """

    _attribute_map = {
        "can_delete": {"key": "deleteEnabled", "type": "bool"},
        "can_write": {"key": "writeEnabled", "type": "bool"},
        "can_list": {"key": "listEnabled", "type": "bool"},
        "can_read": {"key": "readEnabled", "type": "bool"},
    }

    def __init__(
        self,
        *,
        can_delete: Optional[bool] = None,
        can_write: Optional[bool] = None,
        can_list: Optional[bool] = None,
        can_read: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword can_delete: Delete enabled.
        :paramtype can_delete: bool
        :keyword can_write: Write enabled.
        :paramtype can_write: bool
        :keyword can_list: List enabled.
        :paramtype can_list: bool
        :keyword can_read: Read enabled.
        :paramtype can_read: bool
        """
        super().__init__(**kwargs)
        self.can_delete = can_delete
        self.can_write = can_write
        self.can_list = can_list
        self.can_read = can_read


class OCIIndex(Manifest):
    """Returns the requested OCI index file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar manifests: List of OCI image layer information.
    :vartype manifests: list[~container_registry.models.ManifestListAttributes]
    :ivar annotations: Additional information provided through arbitrary metadata.
    :vartype annotations: ~container_registry.models.Annotations
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "manifests": {"key": "manifests", "type": "[ManifestListAttributes]"},
        "annotations": {"key": "annotations", "type": "Annotations"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        manifests: Optional[List["_models.ManifestListAttributes"]] = None,
        annotations: Optional["_models.Annotations"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword manifests: List of OCI image layer information.
        :paramtype manifests: list[~container_registry.models.ManifestListAttributes]
        :keyword annotations: Additional information provided through arbitrary metadata.
        :paramtype annotations: ~container_registry.models.Annotations
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.manifests = manifests
        self.annotations = annotations


class OCIManifest(Manifest):
    """Returns the requested OCI Manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar config: V2 image config descriptor.
    :vartype config: ~container_registry.models.Descriptor
    :ivar layers: List of V2 image layer information.
    :vartype layers: list[~container_registry.models.Descriptor]
    :ivar annotations: Additional information provided through arbitrary metadata.
    :vartype annotations: ~container_registry.models.Annotations
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "config": {"key": "config", "type": "Descriptor"},
        "layers": {"key": "layers", "type": "[Descriptor]"},
        "annotations": {"key": "annotations", "type": "Annotations"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        config: Optional["_models.Descriptor"] = None,
        layers: Optional[List["_models.Descriptor"]] = None,
        annotations: Optional["_models.Annotations"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword config: V2 image config descriptor.
        :paramtype config: ~container_registry.models.Descriptor
        :keyword layers: List of V2 image layer information.
        :paramtype layers: list[~container_registry.models.Descriptor]
        :keyword annotations: Additional information provided through arbitrary metadata.
        :paramtype annotations: ~container_registry.models.Annotations
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.config = config
        self.layers = layers
        self.annotations = annotations


class Paths108HwamOauth2ExchangePostRequestbodyContentApplicationXWwwFormUrlencodedSchema(_serialization.Model):
    """Paths108HwamOauth2ExchangePostRequestbodyContentApplicationXWwwFormUrlencodedSchema.

    All required parameters must be populated in order to send to Azure.

    :ivar grant_type: Can take a value of access_token_refresh_token, or access_token, or
     refresh_token. Required. Known values are: "access_token_refresh_token", "access_token", and
     "refresh_token".
    :vartype grant_type: str or ~container_registry.models.PostContentSchemaGrantType
    :ivar service: Indicates the name of your Azure container registry. Required.
    :vartype service: str
    :ivar tenant: AAD tenant associated to the AAD credentials.
    :vartype tenant: str
    :ivar refresh_token: AAD refresh token, mandatory when grant_type is access_token_refresh_token
     or refresh_token.
    :vartype refresh_token: str
    :ivar aad_access_token: AAD access token, mandatory when grant_type is
     access_token_refresh_token or access_token.
    :vartype aad_access_token: str
    """

    _validation = {
        "grant_type": {"required": True},
        "service": {"required": True},
    }

    _attribute_map = {
        "grant_type": {"key": "grant_type", "type": "str"},
        "service": {"key": "service", "type": "str"},
        "tenant": {"key": "tenant", "type": "str"},
        "refresh_token": {"key": "refresh_token", "type": "str"},
        "aad_access_token": {"key": "access_token", "type": "str"},
    }

    def __init__(
        self,
        *,
        grant_type: Union[str, "_models.PostContentSchemaGrantType"],
        service: str,
        tenant: Optional[str] = None,
        refresh_token: Optional[str] = None,
        aad_access_token: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword grant_type: Can take a value of access_token_refresh_token, or access_token, or
         refresh_token. Required. Known values are: "access_token_refresh_token", "access_token", and
         "refresh_token".
        :paramtype grant_type: str or ~container_registry.models.PostContentSchemaGrantType
        :keyword service: Indicates the name of your Azure container registry. Required.
        :paramtype service: str
        :keyword tenant: AAD tenant associated to the AAD credentials.
        :paramtype tenant: str
        :keyword refresh_token: AAD refresh token, mandatory when grant_type is
         access_token_refresh_token or refresh_token.
        :paramtype refresh_token: str
        :keyword aad_access_token: AAD access token, mandatory when grant_type is
         access_token_refresh_token or access_token.
        :paramtype aad_access_token: str
        """
        super().__init__(**kwargs)
        self.grant_type = grant_type
        self.service = service
        self.tenant = tenant
        self.refresh_token = refresh_token
        self.aad_access_token = aad_access_token


class PathsV3R3RxOauth2TokenPostRequestbodyContentApplicationXWwwFormUrlencodedSchema(_serialization.Model):
    """PathsV3R3RxOauth2TokenPostRequestbodyContentApplicationXWwwFormUrlencodedSchema.

    All required parameters must be populated in order to send to Azure.

    :ivar service: Indicates the name of your Azure container registry. Required.
    :vartype service: str
    :ivar scope: Which is expected to be a valid scope, and can be specified more than once for
     multiple scope requests. You obtained this from the Www-Authenticate response header from the
     challenge. Required.
    :vartype scope: str
    :ivar acr_refresh_token: Must be a valid ACR refresh token. Required.
    :vartype acr_refresh_token: str
    :ivar grant_type: Grant type is expected to be refresh_token. Known values are: "refresh_token"
     and "password".
    :vartype grant_type: str or ~container_registry.models.TokenGrantType
    """

    _validation = {
        "service": {"required": True},
        "scope": {"required": True},
        "acr_refresh_token": {"required": True},
        "grant_type": {"required": True},
    }

    _attribute_map = {
        "service": {"key": "service", "type": "str"},
        "scope": {"key": "scope", "type": "str"},
        "acr_refresh_token": {"key": "refresh_token", "type": "str"},
        "grant_type": {"key": "grant_type", "type": "str"},
    }

    def __init__(
        self,
        *,
        service: str,
        scope: str,
        acr_refresh_token: str,
        grant_type: Union[str, "_models.TokenGrantType"] = "refresh_token",
        **kwargs: Any
    ) -> None:
        """
        :keyword service: Indicates the name of your Azure container registry. Required.
        :paramtype service: str
        :keyword scope: Which is expected to be a valid scope, and can be specified more than once for
         multiple scope requests. You obtained this from the Www-Authenticate response header from the
         challenge. Required.
        :paramtype scope: str
        :keyword acr_refresh_token: Must be a valid ACR refresh token. Required.
        :paramtype acr_refresh_token: str
        :keyword grant_type: Grant type is expected to be refresh_token. Known values are:
         "refresh_token" and "password".
        :paramtype grant_type: str or ~container_registry.models.TokenGrantType
        """
        super().__init__(**kwargs)
        self.service = service
        self.scope = scope
        self.acr_refresh_token = acr_refresh_token
        self.grant_type = grant_type


class Platform(_serialization.Model):
    """The platform object describes the platform which the image in the manifest runs on. A full list
    of valid operating system and architecture values are listed in the Go language documentation
    for $GOOS and $GOARCH.

    :ivar architecture: Specifies the CPU architecture, for example amd64 or ppc64le.
    :vartype architecture: str
    :ivar os: The os field specifies the operating system, for example linux or windows.
    :vartype os: str
    :ivar os_version: The optional os.version field specifies the operating system version, for
     example 10.0.10586.
    :vartype os_version: str
    :ivar os_features: The optional os.features field specifies an array of strings, each listing a
     required OS feature (for example on Windows win32k.
    :vartype os_features: list[str]
    :ivar variant: The optional variant field specifies a variant of the CPU, for example armv6l to
     specify a particular CPU variant of the ARM CPU.
    :vartype variant: str
    :ivar features: The optional features field specifies an array of strings, each listing a
     required CPU feature (for example sse4 or aes.
    :vartype features: list[str]
    """

    _attribute_map = {
        "architecture": {"key": "architecture", "type": "str"},
        "os": {"key": "os", "type": "str"},
        "os_version": {"key": "os\\.version", "type": "str"},
        "os_features": {"key": "os\\.features", "type": "[str]"},
        "variant": {"key": "variant", "type": "str"},
        "features": {"key": "features", "type": "[str]"},
    }

    def __init__(
        self,
        *,
        architecture: Optional[str] = None,
        os: Optional[str] = None,
        os_version: Optional[str] = None,
        os_features: Optional[List[str]] = None,
        variant: Optional[str] = None,
        features: Optional[List[str]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword architecture: Specifies the CPU architecture, for example amd64 or ppc64le.
        :paramtype architecture: str
        :keyword os: The os field specifies the operating system, for example linux or windows.
        :paramtype os: str
        :keyword os_version: The optional os.version field specifies the operating system version, for
         example 10.0.10586.
        :paramtype os_version: str
        :keyword os_features: The optional os.features field specifies an array of strings, each
         listing a required OS feature (for example on Windows win32k.
        :paramtype os_features: list[str]
        :keyword variant: The optional variant field specifies a variant of the CPU, for example armv6l
         to specify a particular CPU variant of the ARM CPU.
        :paramtype variant: str
        :keyword features: The optional features field specifies an array of strings, each listing a
         required CPU feature (for example sse4 or aes.
        :paramtype features: list[str]
        """
        super().__init__(**kwargs)
        self.architecture = architecture
        self.os = os
        self.os_version = os_version
        self.os_features = os_features
        self.variant = variant
        self.features = features


class Repositories(_serialization.Model):
    """List of repositories.

    :ivar repositories: Repository names.
    :vartype repositories: list[str]
    :ivar link:
    :vartype link: str
    """

    _attribute_map = {
        "repositories": {"key": "repositories", "type": "[str]"},
        "link": {"key": "link", "type": "str"},
    }

    def __init__(self, *, repositories: Optional[List[str]] = None, link: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword repositories: Repository names.
        :paramtype repositories: list[str]
        :keyword link:
        :paramtype link: str
        """
        super().__init__(**kwargs)
        self.repositories = repositories
        self.link = link


class RepositoryTags(_serialization.Model):
    """Result of the request to list tags of the image.

    :ivar name: Name of the image.
    :vartype name: str
    :ivar tags: List of tags.
    :vartype tags: list[str]
    """

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "tags": {"key": "tags", "type": "[str]"},
    }

    def __init__(self, *, name: Optional[str] = None, tags: Optional[List[str]] = None, **kwargs: Any) -> None:
        """
        :keyword name: Name of the image.
        :paramtype name: str
        :keyword tags: List of tags.
        :paramtype tags: list[str]
        """
        super().__init__(**kwargs)
        self.name = name
        self.tags = tags


class RepositoryWriteableProperties(_serialization.Model):
    """Changeable attributes for Repository.

    :ivar can_delete: Delete enabled.
    :vartype can_delete: bool
    :ivar can_write: Write enabled.
    :vartype can_write: bool
    :ivar can_list: List enabled.
    :vartype can_list: bool
    :ivar can_read: Read enabled.
    :vartype can_read: bool
    """

    _attribute_map = {
        "can_delete": {"key": "deleteEnabled", "type": "bool"},
        "can_write": {"key": "writeEnabled", "type": "bool"},
        "can_list": {"key": "listEnabled", "type": "bool"},
        "can_read": {"key": "readEnabled", "type": "bool"},
    }

    def __init__(
        self,
        *,
        can_delete: Optional[bool] = None,
        can_write: Optional[bool] = None,
        can_list: Optional[bool] = None,
        can_read: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword can_delete: Delete enabled.
        :paramtype can_delete: bool
        :keyword can_write: Write enabled.
        :paramtype can_write: bool
        :keyword can_list: List enabled.
        :paramtype can_list: bool
        :keyword can_read: Read enabled.
        :paramtype can_read: bool
        """
        super().__init__(**kwargs)
        self.can_delete = can_delete
        self.can_write = can_write
        self.can_list = can_list
        self.can_read = can_read


class TagAttributesBase(_serialization.Model):
    """Tag attribute details.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Tag name. Required.
    :vartype name: str
    :ivar digest: Tag digest. Required.
    :vartype digest: str
    :ivar created_on: Tag created time. Required.
    :vartype created_on: ~datetime.datetime
    :ivar last_updated_on: Tag last update time. Required.
    :vartype last_updated_on: ~datetime.datetime
    :ivar changeable_attributes: Writeable properties of the resource. Required.
    :vartype changeable_attributes: ~container_registry.models.TagWriteableProperties
    """

    _validation = {
        "name": {"required": True, "readonly": True},
        "digest": {"required": True, "readonly": True},
        "created_on": {"required": True, "readonly": True},
        "last_updated_on": {"required": True, "readonly": True},
        "changeable_attributes": {"required": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "digest": {"key": "digest", "type": "str"},
        "created_on": {"key": "createdTime", "type": "iso-8601"},
        "last_updated_on": {"key": "lastUpdateTime", "type": "iso-8601"},
        "changeable_attributes": {"key": "changeableAttributes", "type": "TagWriteableProperties"},
    }

    def __init__(self, *, changeable_attributes: "_models.TagWriteableProperties", **kwargs: Any) -> None:
        """
        :keyword changeable_attributes: Writeable properties of the resource. Required.
        :paramtype changeable_attributes: ~container_registry.models.TagWriteableProperties
        """
        super().__init__(**kwargs)
        self.name = None
        self.digest = None
        self.created_on = None
        self.last_updated_on = None
        self.changeable_attributes = changeable_attributes


class TagAttributesTag(_serialization.Model):
    """Tag.

    :ivar signature_record: SignatureRecord value.
    :vartype signature_record: str
    """

    _attribute_map = {
        "signature_record": {"key": "signatureRecord", "type": "str"},
    }

    def __init__(self, *, signature_record: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword signature_record: SignatureRecord value.
        :paramtype signature_record: str
        """
        super().__init__(**kwargs)
        self.signature_record = signature_record


class TagList(_serialization.Model):
    """List of tag details.

    All required parameters must be populated in order to send to Azure.

    :ivar registry_login_server: Registry login server name. This is likely to be similar to
     {registry-name}.azurecr.io. Required.
    :vartype registry_login_server: str
    :ivar repository: Image name. Required.
    :vartype repository: str
    :ivar tag_attribute_bases: List of tag attribute details. Required.
    :vartype tag_attribute_bases: list[~container_registry.models.TagAttributesBase]
    :ivar link:
    :vartype link: str
    """

    _validation = {
        "registry_login_server": {"required": True},
        "repository": {"required": True},
        "tag_attribute_bases": {"required": True},
    }

    _attribute_map = {
        "registry_login_server": {"key": "registry", "type": "str"},
        "repository": {"key": "imageName", "type": "str"},
        "tag_attribute_bases": {"key": "tags", "type": "[TagAttributesBase]"},
        "link": {"key": "link", "type": "str"},
    }

    def __init__(
        self,
        *,
        registry_login_server: str,
        repository: str,
        tag_attribute_bases: List["_models.TagAttributesBase"],
        link: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword registry_login_server: Registry login server name. This is likely to be similar to
         {registry-name}.azurecr.io. Required.
        :paramtype registry_login_server: str
        :keyword repository: Image name. Required.
        :paramtype repository: str
        :keyword tag_attribute_bases: List of tag attribute details. Required.
        :paramtype tag_attribute_bases: list[~container_registry.models.TagAttributesBase]
        :keyword link:
        :paramtype link: str
        """
        super().__init__(**kwargs)
        self.registry_login_server = registry_login_server
        self.repository = repository
        self.tag_attribute_bases = tag_attribute_bases
        self.link = link


class TagWriteableProperties(_serialization.Model):
    """Changeable attributes.

    :ivar can_delete: Delete enabled.
    :vartype can_delete: bool
    :ivar can_write: Write enabled.
    :vartype can_write: bool
    :ivar can_list: List enabled.
    :vartype can_list: bool
    :ivar can_read: Read enabled.
    :vartype can_read: bool
    """

    _attribute_map = {
        "can_delete": {"key": "deleteEnabled", "type": "bool"},
        "can_write": {"key": "writeEnabled", "type": "bool"},
        "can_list": {"key": "listEnabled", "type": "bool"},
        "can_read": {"key": "readEnabled", "type": "bool"},
    }

    def __init__(
        self,
        *,
        can_delete: Optional[bool] = None,
        can_write: Optional[bool] = None,
        can_list: Optional[bool] = None,
        can_read: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword can_delete: Delete enabled.
        :paramtype can_delete: bool
        :keyword can_write: Write enabled.
        :paramtype can_write: bool
        :keyword can_list: List enabled.
        :paramtype can_list: bool
        :keyword can_read: Read enabled.
        :paramtype can_read: bool
        """
        super().__init__(**kwargs)
        self.can_delete = can_delete
        self.can_write = can_write
        self.can_list = can_list
        self.can_read = can_read


class V1Manifest(Manifest):
    """Returns the requested V1 manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar architecture: CPU architecture.
    :vartype architecture: str
    :ivar name: Image name.
    :vartype name: str
    :ivar tag: Image tag.
    :vartype tag: str
    :ivar fs_layers: List of layer information.
    :vartype fs_layers: list[~container_registry.models.FsLayer]
    :ivar history: Image history.
    :vartype history: list[~container_registry.models.History]
    :ivar signatures: Image signature.
    :vartype signatures: list[~container_registry.models.ImageSignature]
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "architecture": {"key": "architecture", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "tag": {"key": "tag", "type": "str"},
        "fs_layers": {"key": "fsLayers", "type": "[FsLayer]"},
        "history": {"key": "history", "type": "[History]"},
        "signatures": {"key": "signatures", "type": "[ImageSignature]"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        architecture: Optional[str] = None,
        name: Optional[str] = None,
        tag: Optional[str] = None,
        fs_layers: Optional[List["_models.FsLayer"]] = None,
        history: Optional[List["_models.History"]] = None,
        signatures: Optional[List["_models.ImageSignature"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword architecture: CPU architecture.
        :paramtype architecture: str
        :keyword name: Image name.
        :paramtype name: str
        :keyword tag: Image tag.
        :paramtype tag: str
        :keyword fs_layers: List of layer information.
        :paramtype fs_layers: list[~container_registry.models.FsLayer]
        :keyword history: Image history.
        :paramtype history: list[~container_registry.models.History]
        :keyword signatures: Image signature.
        :paramtype signatures: list[~container_registry.models.ImageSignature]
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.architecture = architecture
        self.name = name
        self.tag = tag
        self.fs_layers = fs_layers
        self.history = history
        self.signatures = signatures


class V2Manifest(Manifest):
    """Returns the requested Docker V2 Manifest file.

    :ivar schema_version: Schema version.
    :vartype schema_version: int
    :ivar media_type: Media type for this Manifest.
    :vartype media_type: str
    :ivar config: V2 image config descriptor.
    :vartype config: ~container_registry.models.Descriptor
    :ivar layers: List of V2 image layer information.
    :vartype layers: list[~container_registry.models.Descriptor]
    """

    _attribute_map = {
        "schema_version": {"key": "schemaVersion", "type": "int"},
        "media_type": {"key": "mediaType", "type": "str"},
        "config": {"key": "config", "type": "Descriptor"},
        "layers": {"key": "layers", "type": "[Descriptor]"},
    }

    def __init__(
        self,
        *,
        schema_version: Optional[int] = None,
        media_type: Optional[str] = None,
        config: Optional["_models.Descriptor"] = None,
        layers: Optional[List["_models.Descriptor"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword schema_version: Schema version.
        :paramtype schema_version: int
        :keyword media_type: Media type for this Manifest.
        :paramtype media_type: str
        :keyword config: V2 image config descriptor.
        :paramtype config: ~container_registry.models.Descriptor
        :keyword layers: List of V2 image layer information.
        :paramtype layers: list[~container_registry.models.Descriptor]
        """
        super().__init__(schema_version=schema_version, **kwargs)
        self.media_type = media_type
        self.config = config
        self.layers = layers
